#!/usr/bin/perl
package Play_GST;
use strict;
use warnings;

my $GST_ok;
my ($PlayBin,$Sink);
my ($WatchTag,$Skip);
my ($Mute,$Volume);
my %Plugins; my %Sinks;

my $reg; #FIXME work around to keep the register from being finalized (see http://bugzilla.gnome.org/show_bug.cgi?id=324818 )

BEGIN
{ %Plugins=(	mp3	=> 'mad',	ogg => 'vorbis',
		flac	=> 'flac',	mpc => 'musepack',
	);
  $Sinks{$_}=undef for qw/autoaudio oss esd alsa artsd/;
  my $error;
  if (grep -f $_.'/GStreamer.pm',@INC)
  {	eval {require GStreamer};
	die "gst0.08\n" if !$@ && GStreamer->VERSION<.05;
	if (!$@ && GStreamer->init_check)
	{	GStreamer->init;
		$reg=GStreamer::Registry->get_default;
		if ( $reg->find_plugin('playbin') ) { $GST_ok=1; }
		else { $error="gstreamer plugin 'playbin' not found\nYou need to install at least gst-plugins-base\n"; }
	}
	else { $error=$@? $@ : "Can't initialize GStreamer.\n"; }
  }
  else {$error="GStreamer.pm not found\n";}
  if ($error) {warn "$error -> gstreamer output won't be available.\n"}
}

sub supported_formats
{	return '' unless $GST_ok;
	#my $reg;#FIXME (see http://bugzilla.gnome.org/show_bug.cgi?id=324818 )
	return grep $reg->find_plugin($Plugins{$_}), keys %Plugins;
}
sub supported_sinks
{	return () unless $GST_ok;
	$Sinks{$_}= $reg->lookup_feature($_.'sink') for keys %Sinks;
	$::Options{gst_sink}='autoaudio' unless $Sinks{$::Options{gst_sink}};
	return grep $Sinks{$_}, keys %Sinks;
}

sub init
{	return undef unless $GST_ok;
	$PlayBin=GStreamer::ElementFactory -> make(playbin => 'playbin');
	my $bus=$PlayBin->get_bus;
	$bus->add_signal_watch;
	$bus->signal_connect(message => \&bus_message);
	$Volume=100 unless defined $Volume;
	1;
}

sub bus_message
{	my $msg=$_[1];
	warn 'bus: message='.$msg->type."\n" if $::debug;
	SkipTo(undef,$Skip) if $Skip && $msg->type & 'state_changed';
	return unless $msg->type & ['eos','error'];
	if ($msg->type & 'error') {::ErrorPlay($msg->error)}
	else { ::end_of_file(); }
}

sub GetVolume	{$Volume}
sub GetMute	{$Mute}
sub SetVolume
{	shift;
	my $set=shift;
	if	($set eq 'mute')	{ $Mute=$Volume; $Volume=0;	}
	elsif	($set eq 'unmute')	{ $Volume=$Mute; $Mute=0;	}
	elsif	($set=~m/^(\d+)%\+$/)	{ $Volume+=$1; }
	elsif	($set=~m/^(\d+)%\-$/)	{ $Volume-=$1; }
	elsif	($set=~m/^(\d+)%$/)	{ $Volume =$1; }
	$Volume=0   if $Volume<0;
	$Volume=100 if $Volume>100;
	$PlayBin->set(volume => $Volume/100);
	::HasChanged('Vol');
}

sub SkipTo
{	shift;
	$Skip=shift;
	my ($result,$state,$pending)=$PlayBin->get_state(0);
	return if $result eq 'async';
	$PlayBin->seek(1,'time','flush','set', $Skip*1_000_000_000,'none',0);($result,$state,$pending)=$PlayBin->get_state(0);
	$Skip=undef;
}

sub Pause
{	$PlayBin->set_state('paused');
}
sub Resume
{	$PlayBin->set_state('playing');
}

sub Play
{	(undef,my$file,$Skip)=@_;
	&Stop;
	#my ($ext)=$file=~m/\.([^.]*)$/; warn $ext;
	#::ErrorPlay('not supported') and return undef  unless $Plugins{$ext};
	$PlayBin->set(volume => $Volume/100);
	$Sink=undef if $Sink && $Sink->get_name ne $::Options{gst_sink};
	unless ($Sink)
	{	$Sink=GStreamer::ElementFactory->make($::Options{gst_sink}.'sink' => $::Options{gst_sink});
		#$Sink->set(device => "/dev/dsp1") if $::Options{gst_sink} eq 'oss'; #TEST
		$PlayBin->set('audio-sink' => $Sink);
	}
	warn "playing $file\n";
	my $f=$file;
	$f=~s#([^A-Za-z0-9- /\.])#sprintf('%%%02X', ord($1))#seg;
	$PlayBin -> set(uri => 'file://'.$f);
	$PlayBin -> set_state('playing');
	$WatchTag=Glib::Timeout->add(500,\&_UpdateTime) unless $WatchTag;
}

sub _UpdateTime
{	my ($result,$state,$pending)=$PlayBin->get_state(0);
	warn "state: $result,$state,$pending\n" if $::debug;
	return 1 if $result eq 'async';
	if ($state ne 'playing' && $state ne 'paused')
	{	return 1 if $pending eq 'playing' || $pending eq 'paused';
		::ResetTime() unless $::Play_package ne 'Play_GST';
		$WatchTag=undef;
		return 0;
	}
	my $query=GStreamer::Query::Position->new('time');
	if ($PlayBin->query($query))
	{	my (undef, $position)=$query->position;
		::UpdateTime( $position/1_000_000_000 );
	}
	return 1;
}

sub Stop
{	my ($result,$state,$pending)=$PlayBin->get_state(0);
	#warn "stop: state: $result,$state,$pending\n";
	return if $state eq 'null' and $pending eq 'void-pending';
	$PlayBin -> set_state('null');
}


1;
