#!/usr/bin/perl
package Play_GST;
use strict;
use warnings;

my $GST_ok;
my $GstThread;
my $GstSink;
my $WatchTag;
my ($Mute,$Volume);
my %Plugins;

BEGIN
{ %Plugins=(	mp3	=> 'mad',	ogg => 'vorbis',
		flac	=> 'flac',	mpc => 'musepack',
	);
  if (grep -f $_."/GStreamer.pm",@INC)
  {	require GStreamer;
	if (GStreamer->init_check)
	{	GStreamer->init;
		$GST_ok=1;
	}
	else { warn "Can't initialise GStreamer.\n"; }
  }
  else {warn "Gstreamer.pm not found -> Gstreamer output won't be available.\n";}
}

sub supported_formats
{	return '' unless $GST_ok;
	return grep GStreamer::RegistryPool->find_plugin($Plugins{$_}), keys %Plugins;
}
sub supported_sinks
{	return () unless $GST_ok;
	my %plugin=(oss => 'ossaudio', alsa => 'alsa', esd => 'esdsink', 'artsd' => 'artsdsink');
	return grep GStreamer::RegistryPool->find_plugin($plugin{$_}), keys %plugin;
}

sub init
{	return undef unless $GST_ok;
	$GstThread=GStreamer::Thread->new('player');
	$GstThread->signal_connect(eos	 => \&_eos_cb);
#	$GstThread->signal_connect(error => \&_error_cb);
	$GstThread->signal_connect(error => sub{warn 'GStreamer error : '.$_[2]->message."\n";});
	$Volume=100 unless defined $Volume;
	1;
}

sub _error_cb
{	my ($GstThread, $element, $error) = @_;
	warn 'GStreamer error : '.$error -> message();
	my $msg='GStreamer error : '.$error -> message();
	::IdleDo('0_gsterror',1,\&::ErrorPlay,$msg , 1);
#	::IdleDo('0_gsterror',1,sub {::ErrorPlay($msg , 1); Stop(); init(); });
	#::IdleDo('0_gsterror',1,\&init);
	return 1;
}

sub _eos_cb
{	Glib::Source->remove($WatchTag) if $WatchTag; $WatchTag=undef;
	$GstThread->set_state('null')  or warn "gstreamer : Could not stop playing";
	$GstThread->remove($GstThread->get_list);
	::IdleDo('0_play',1,\&::end_of_file);
}

sub GetVolume	{$Volume}
sub GetMute	{$Mute}
sub SetVolume
{	shift;
	my $set=shift;
	if	($set eq 'mute')	{ $Mute=$Volume; $Volume=0;	}
	elsif	($set eq 'unmute')	{ $Volume=$Mute; $Mute=0;	}
	elsif	($set=~m/^(\d+)%\+$/)	{ $Volume+=$1; }
	elsif	($set=~m/^(\d+)%\-$/)	{ $Volume-=$1; }
	elsif	($set=~m/^(\d+)%$/)	{ $Volume =$1; }
	$Volume=0   if $Volume<0;
	$Volume=100 if $Volume>100;
	if ($GstThread->get_state >= 'playing')
	{	$GstThread->get_by_name('volume')->set(volume => $Volume/100);
	}
	::HasChanged('Vol');
}

sub SkipTo
{	shift;
	my $sec=shift;
	$GstThread -> set_state('playing') unless $GstThread->get_state >= 'playing';
	$GstSink->seek(['method-set','time','flag-flush'], $sec*1_000_000_000);
}

sub Pause
{	$GstThread -> set_state('paused');
}
sub Resume
{	$GstThread -> set_state('playing') or die "Could not restart playing";
}

sub Play
{	my (undef,$file,$sec)=@_;
	&Stop if $GstThread->get_state >= 'playing';
#	init();	#TEST to prevent crash
	#my ($ext)=$file=~m/\.([^.]*)$/; warn $ext;
	#::ErrorPlay('not supported') and return undef  unless $Plugins{$ext} && ;
	my ($source, $spider, $volume, $sink) =
	GStreamer::ElementFactory -> make(
		filesrc=> 'src',
		spider => 'spider', #FIXME : use decodebin
		volume => 'volume',
		$::Options{gst_sink}.'sink' => 'sink');
#		'audioconvert'	=> 'converter',
#		'audioscale'	=> 'scale',
	$GstSink=($::Options{gst_sink} eq 'autoaudio')? $sink->get_list : $sink;
	$volume->set(volume => $Volume/100);
	warn "playing $file\n";
	$source -> set(location => $file);
	$GstThread -> add($source, $spider, $volume, $sink);
	$source -> link($spider, $volume,$sink) or die "Could not link";
	$GstThread -> set_state('playing') or die "Could not start playing";
	SkipTo(undef,$sec) if $sec;
	$WatchTag=Glib::Timeout->add(500,\&_UpdateTime);
}

sub _UpdateTime
{	#warn $GstThread->get_state;
	unless ($GstThread->get_state & ['playing','paused'])
	{	return 1 if $::ToDo{'0_play'};
		::ResetTime();
		return 0;
	}
	::UpdateTime( $GstSink->query(position => 'time')/1000000000 );
	return 1;
}

sub Stop
{	Glib::Source->remove($WatchTag) if $WatchTag; $WatchTag=undef;
	return if ($GstThread->get_state >= 'null');
	$GstThread -> set_state('null')  or die "Could not stop playing";
#	warn join(' ','before:',$GstThread->get_list)."\n";
	$GstThread->remove($GstThread->get_list);
#	warn join(' ','after:',$GstThread->get_list)."\n";
}


1;
