package Player;
use strict;
use Gtk2;

use constant
{
 TRUE  => 1,
 FALSE => 0,

 SIZE_BUTTONS => 'large-toolbar',
 SIZE_FLAGS => 'menu',
};

my @MenuQueue=
(	{label => _"Queue album",	code => sub { ::EnqueueAlbum($_[0]{ID}); } },
	{label => _"Queue artist",	code => sub { ::EnqueueArtist($_[0]{ID});} },
	{label => _"Auto fill queue",	code => sub {&::EnqueueAction('autofill')},	check => sub {$::QueueAction eq 'autofill'} },
	{label => _"Wait when queue empty",	code => sub {&::EnqueueAction('wait')}, check => sub {$::QueueAction eq 'wait'} },
	{label => _"Stop when queue empty",	code => sub {&::EnqueueAction('stop')}, check => sub {$::QueueAction eq 'stop'} },
	{label => _"Quit when queue empty",	code => sub {&::EnqueueAction('quit')}, check => sub {$::QueueAction eq 'quit'} },
	{label => _"Clear queue",	code => \&::ClearQueue,		test => sub{@::Queue}},
	{label => _"Shuffle queue",	code => \&::ShuffleQueue,	test => sub{@::Queue}},
	{label => _"Edit...",		code => \&::EditQueue},
);

my @MainMenu=
(	{label => _"Settings",		code => \&::PrefDialog,	stockicon => 'gtk-preferences' },
	{label => _"Open Browser",	code => \&::Playlist,	stockicon => 'gmb-playlist' },
	{label => _"Open Context window",code => \&::ContextWindow, stockicon => 'gtk-info'},
	{label => _"About",		code => \&::AboutDialog,stockicon => 'gtk-about' },
	{label => _"Quit",		code => \&::Quit,	stockicon => 'gtk-quit' },
);

my %objects=
(	Prev =>
	{	class	=> 'Player::Button',
		size	=> SIZE_BUTTONS,
		pic	=> ['gtk-media-previous'],
		tip	=> _"Recently played songs",
		activate=> \&::PrevSong,
		click3	=> sub { ::ChooseSongs(undef,::GetPrevSongs(5)); },
	},
	Stop =>
	{	class	=> 'Player::Button',
		size	=> SIZE_BUTTONS,
		pic	=> ['gtk-media-stop'],
		tip	=> _"Stop",
		activate=> \&::Stop,
		click3	=> sub {&::EnqueueAction('stop')},
	},
	Play =>
	{	class	=> 'Player::Button',
		size	=> SIZE_BUTTONS,
		pic	=> sub {$::TogPlay? ['gtk-media-pause'] : ['gtk-media-play']},
		tip	=> sub {$::TogPlay? _"Pause" : _"Play"},
		activate=> \&::PlayPause,
		#click3	=> undef,
		event	=> 'Playing',
	},
	Next =>
	{	class	=> 'Player::Button',
		size	=> SIZE_BUTTONS,
		pic	=> ['gtk-media-next'],
		tip	=> _"Next",
		activate=> \&::NextSong,
		click3	=> sub { ::ChooseSongs(undef,::GetNextSongs(5)); },
	},
	Playlist =>
	{	class	=> 'Player::Button',
		size	=> SIZE_BUTTONS,
		pic	=> ['gmb-playlist'],
		tip	=> _"Open Browser window",
		activate=> sub {::Playlist($_[0]->{extra})},
		click3	=> sub {Player->new($::Options{LayoutB});},
	},
	BContext =>
	{	class	=> 'Player::Button',
		size	=> SIZE_BUTTONS,
		pic	=> ['gtk-info'],
		tip	=> _"Open Context window",
		activate=> sub {::ContextWindow($_[0]->{extra})},
		click3	=> sub {Player->new('Context');},
	},
	Pref =>
	{	class	=> 'Player::Button',
		size	=> SIZE_BUTTONS,
		pic	=> ['gtk-preferences'],
		tip	=> _"Edit Settings",
		activate=> \&::PrefDialog,
		click3	=> sub {Player->new($::Options{Layout});},
		click2	=> \&::AboutDialog,
	},
	Quit =>
	{	class	=> 'Player::Button',
		size	=> SIZE_BUTTONS,
		pic	=> ['gtk-quit'],
		tip	=> _"Quit",
		activate=> \&::Quit,
		click3	=> sub {&::EnqueueAction('quit')},
	},
	LockArtist =>
	{	class	=> 'Player::Flag',
		size	=> SIZE_FLAGS,
		pic	=> sub { ($::TogLock==::SONG_ARTIST)? ['gmb-lock'] :  ['gmb-empty','gmb-locklight'] },
		tip	=> _"Lock On Artist",
		click1	=> sub {::ToggleLock(::SONG_ARTIST);},
		event	=> 'Lock',
	},
	LockAlbum =>
	{	class	=> 'Player::Flag',
		size	=> SIZE_FLAGS,
		pic	=> sub { ($::TogLock==::SONG_ALBUM)? ['gmb-lock'] :  ['gmb-empty','gmb-locklight'] },
		tip	=> _"Lock On Album",
		click1	=> sub {::ToggleLock(::SONG_ALBUM);},
		event	=> 'Lock',
	},
	Sort =>
	{	class	=> 'Player::Flag',
		size	=> SIZE_FLAGS,
		pic	=> sub { my $s=$::Options{'Sort'};($s=~/^r/)? 'gmb-random' : ($s eq 's')? 'gmb-shuffle' : 'gtk-sort-ascending'; },
		tip	=> sub { _("Play order :\n").::ExplainSort($::Options{'Sort'}); },
		click1	=> \&::ToggleSort,
		click3	=> \&SortMenu,
		event	=> [qw/Sort SavedWRandoms SavedSorts/],
	},
	Filter =>
	{	class	=> 'Player::Flag',
		size	=> SIZE_FLAGS,
		pic	=> sub { defined $::ListMode ? 'gmb-list'
			: $::SelectedFilter->is_empty ? 'gmb-library' : 'gmb-filter'; },
		tip	=> sub
			{ defined $::ListMode	? ref $::ListMode	? _"unnamed static list"
									: $::ListMode.' list'
						: _("Playlist filter :\n").$::SelectedFilter->explain;
			},
		click1	=> \&RemoveFilter,
		click3	=> \&FilterMenu,
		event	=> [qw/Filter SavedFilters/],
	},
	Queue =>
	{	class	=> 'Player::Flag',
		size	=> SIZE_FLAGS,
		pic	=> sub { @::Queue?	 ['gmb-queue'] :
				 $::QueueAction? [$::QActions{$::QueueAction}[0]] :
				 		 ['gmb-empty','gmb-queue'] },
		tip	=> sub { ::CalcListLength(\@::Queue,'queue')
				.($::QueueAction? "\n". ::__x( _"then {action}", action => $::QActions{$::QueueAction}[1] ) : '');
				},
		click1	=> \&::ClearQueue,
		click3	=> sub {::PopupContextMenu(\@MenuQueue,{ID=>$::SongID});},
		event	=> 'Queue',
		dragdest=> [::DRAG_ID,sub {shift;shift;::Enqueue(@_);}],
	},
	Vol =>
	{	class	=> 'Player::Flag',
		size	=> SIZE_BUTTONS,
		pic	=> sub { 'gmb-vol'.( ::GetMute() ? 'm' : int((::GetVol()-1)/100*$::NBVolIcons) ); },
		tip	=> sub { _("Volume : ").::GetVol().'%' },
		click1	=> sub { ::PopupVol() },
		click3	=> sub { ::ChangeVol('mute') },
		event	=> 'Vol',
	},
	Pos =>
	{	class	=> 'Player::Label',
		click1	=> sub { ::ChooseSongs(undef,::GetNeighbourSongs(5)) unless $::RandomMode || @::Queue; },
		update	=> sub  { my $t=(@::ListPlay==0)?	'':
					@::Queue	?	::__("%d song in queue","%d songs in queue",scalar@::Queue):
					$::RandomMode	?	::__("%d song","%d songs",scalar@::ListPlay):
								($::Position+1).'/'.@::ListPlay;
				  $_[0]->{label}->set_markup( '<small>'.$t.'</small>' );
				},
		event	=> ['Pos','Queue','Sort'],
	},
	Title =>
	{	class	=> 'Player::ScrollingLabel',
		size	=> 20,
		markup	=> '<b><big>%S</big></b>%V',
		markup_empty => '<b><big>&lt;'._("Playlist Empty").'&gt;</big></b>',
		click1	=> sub {::ChooseSongsFromA($::Songs[$::SongID][::SONG_ALBUM]);0; },
		click3	=> sub {::PopupContextMenu(\@::SongCMenu,{mode=> 'P', self=> $_[0], IDs => [$::SongID]}) if defined $::SongID;},
		dragsrc => [::DRAG_ID,sub {$::SongID}],
		dragdest=> [::DRAG_ID,sub {::Select(undef,undef,$_[2]);}],
	},
	Title_by =>
	{	class	=> 'Player::ScrollingLabel',
		size	=> 20,
		markup	=> ::__x(_"{song} by {artist}",song => "<b><big>%S</big></b>%V", artist => "<b>%a</b>"),
		markup_empty => '<b><big>&lt;'._("Playlist Empty").'&gt;</big></b>',
		click1	=> sub {::ChooseSongsFromA($::Songs[$::SongID][::SONG_ALBUM]);0; },
		click3	=> sub {::PopupContextMenu(\@::SongCMenu,{mode=> 'P', self=> $_[0], IDs => [$::SongID]}) if defined $::SongID;},
		dragsrc => [::DRAG_ID,sub {$::SongID}],
	},
	Artist =>
	{	class	=> 'Player::ScrollingLabel',
		size	=> 20,
		markup	=> '<b>%a</b>',
		click1	=> sub { ::PopupAA(::SONG_ARTIST); },
		click3	=> sub { ::ArtistContextMenu($_[0],$::Songs[$::SongID][::SONG_ARTIST],'P') if defined $::SongID; },
		fields	=> ::SONG_ARTIST,
		dragsrc => [::DRAG_ARTIST,sub {$::Songs[$::SongID][::SONG_ARTIST]}],
	},
	Album =>
	{	class	=> 'Player::ScrollingLabel',
		size	=> 20,
		markup	=> '<b>%l</b>',
		click1	=> sub { ::PopupAA(::SONG_ALBUM,$::Songs[$::SongID][::SONG_ARTIST]); },
		click3	=> sub { ::PopupContextMenu(\@::cMenuAA,{self =>$_[0], col=>::SONG_ALBUM, ID=>$::SongID, key=>$::Songs[$::SongID][::SONG_ALBUM], mode => 'P'}) if defined $::SongID; },
		fields	=> ::SONG_ALBUM,
		dragsrc => [::DRAG_ALBUM,sub {$::Songs[$::SongID][::SONG_ALBUM]}],
	},
	Date =>
	{	class	=> 'Player::Label',
		markup	=> ' %y',
		markup_empty=> '',
		fields	=> ::SONG_DATE,
	},
	Comment =>
	{	class	=> 'Player::Label',
		markup	=> '%C',
		fields	=> ::SONG_COMMENT,
	},
	Length =>
	{	class	=> 'Player::Label',
		markup	=> 	::__x( _" of {length}", 'length' => "%m" ),
		markup_empty=>	::__x( _" of {length}", 'length' => "0:00" ),
		fields	=> ::SONG_LENGTH,
#		font	=> 'Monospace',
	},
	LabelTime =>
	{	New	=> sub	{ my $l=Gtk2::Label->new;
				  $l->set_alignment(1,.5);
#				  $l->modify_font(Gtk2::Pango::FontDescription->from_string('Monospace'));
				  $l->set_size_request($l->create_pango_layout('XX:XX')->get_pixel_size);
				  return $l;
			  	},
		event	=> 'Time',
		update	=> sub { $_[0]->set_label($::PlayTimeMMSS) unless $_[0]->{busy}; },
	},
	Scale =>
	{	class	=> 'Player::TimeSlider',
		event	=> 'Time',
		update	=> sub { $_[0]->{adj}->set_value((defined $::PlayTime)? $::PlayTime : 0) unless $_[0]->{pressed}; },
		fields	=> ::SONG_LENGTH,
		schange	=> sub { my $l=(defined $_[1])? $::Songs[$_[1]][::SONG_LENGTH] : 0; $_[0]->{adj}->upper($l); },
	},
	Stars =>
	{	New	=> sub	{ Stars->new(0,sub {	return unless defined $::SongID;
							$::Songs[$::SongID][::SONG_RATING]=$_[0];
							::SongChanged($::SongID,::SONG_RATING);
						  });
				},
		fields	=> ::SONG_RATING,
		schange	=> sub	{ my $r=(defined $_[1])? $::Songs[$_[1]][::SONG_RATING] : 0; $_[0]->set($r); },
	},
	Cover =>
	{	class	=> 'Player::Cover',
		schange	=> sub { my $key=(defined $_[1])? $::Songs[$_[1]][::SONG_ALBUM] : '' ;$_[0]->set($key); },
		click1	=> sub {::ChooseSongsFromA($::Songs[$::SongID][::SONG_ALBUM]);0; },
		event	=> 'AAPicture',
		update	=> \&Player::Cover::AAPicture_Changed,
		#size	=> 60,
		noinit	=> 1,
		dragsrc => [::DRAG_ALBUM,sub {$::Songs[$::SongID][::SONG_ALBUM]}],
	},
	Filler =>
	{	New	=> sub { Gtk2::HBox->new; },
	},
	SongList =>
	{	New	=> sub { SongList->new('B',$_[0],$_[1]); },
		default => '1_11_12_13_0,9_10_11_14_3_5_13_0_20_21_22,519_200_160_50_80_54_100_100_100_254_200_200_50_22_40_100_71_200_100_200_200_50_48_160',	#default sort order,columns,columns width
	},
	AABox	=>
	{	New	=> sub { AABox->new($_[0]); },
	},
	FPane	=>
	{	New	=> sub { FilterPane->new($_[0],$_[1]); },
		#class	=> 'FilterPane',
	},
	Total	=>
	{	New	=> sub { LabelTotal->new( $_[1] ); },
	},
	FBox	=>
	{	New	=> sub { Browser::makeFilterBox(); },
		dragdest => [::DRAG_FILTER,sub {$_[0]->get_toplevel->SetFilter(1,$_[2]);}],
	},
	FLock	=>	{ New => \&Browser::makeLockToggle,},
	HistItem =>	{ New => \&Browser::make_history_menuitem,
			  setmenu => \&Browser::make_history_menu,
			},
	PlayItem =>	{ New => sub { Gtk2::MenuItem->new(_"Playing"); },
			  setmenu => \&Browser::make_playing_menu,
			},
	LSortItem =>	{ New => sub { Gtk2::MenuItem->new(_"Sort"); },
			  setmenu => \&Browser::make_sort_menu,
			},
	PSortItem =>	{ New => sub { Gtk2::MenuItem->new(_"Play order"); },
			  setmenu => sub {SortMenu();},
			},
	PFilterItem =>	{ New => sub { Gtk2::MenuItem->new(_"Playlist filter"); },
			  setmenu => sub {FilterMenu();},
			},
	QueueItem =>	{ New => sub { Gtk2::MenuItem->new(_"Queue"); },
			  setmenu => sub{ my $m=::PopupContextMenu(\@MenuQueue,{ID=>$::SongID}); },
			},
	MainMenuItem =>	{ New => sub { Gtk2::MenuItem->new(_"Main"); },
			  setmenu => sub{ my $m=::PopupContextMenu(\@MainMenu); },
			},
	Label =>	{ New => sub { Gtk2::Label->new($_[0]); }	},
	Refresh =>
	{	class	=> 'Player::Button',
		size	=> 'menu',
		pic	=> ['gtk-refresh'],
		tip	=> _"Refresh list",
		activate=> sub { $_[0]->get_toplevel->UpdateFilter; },
	},
	PlayFilter =>
	{	class	=> 'Player::Button',
		size	=> 'menu',
		pic	=> ['gtk-media-play'],
		tip	=> _"Play filter",
		activate=> sub { ::Select($_[0]->get_toplevel->{Filter},undef,'trykeep','play'); },
	},
	QueueFilter =>
	{	class	=> 'Player::Button',
		size	=> 'menu',
		pic	=> ['gmb-queue'],
		tip	=> _"Enqueue filter",
		activate=> sub { ::EnqueueFilter( $_[0]->get_toplevel->{Filter} ); },
	},
	ResetFilter =>
	{	class	=> 'Player::Button',
		size	=> 'menu',
		pic	=> ['gtk-clear'],
		tip	=> _"Reset filter",
		activate=> sub { $_[0]->get_toplevel->SetFilter(1,undef); },
	},
	VolSlider =>
	{	New	=> \&::makeVolSlider,
		event	=> 'Vol',
		update	=> sub { $_[0]->get_adjustment->set_value(::GetVol()); },
	},
	QueueList =>
	{	New	=> sub {EditList->new(undef,$_[0],$_[1])}
	},
	EditList =>
	{	New	=> sub {EditList->new(1,$_[0],$_[1])}
	},
	Context =>
	{	class	=> 'Player::Context',
	},
	TogButton =>
	{	class	=> 'Player::TogButton',
	},
	HSeparator =>
	{	New	=> sub {Gtk2::HSeparator->new},
	},
	VSeparator =>
	{	New	=> sub {Gtk2::VSeparator->new},
	},
	Choose =>
	{	class	=> 'Player::Button',
		size	=> SIZE_BUTTONS,
		pic	=> ['gtk-add'],
		tip	=> _"Choose Artist/Album/Song",
		activate=> sub { Player->new('Search'); },
	},
	ChooseRandAlbum =>		#TEST
	{	class	=> 'Player::Button',
		size	=> SIZE_BUTTONS,
		pic	=> ['gmb-random'], #FIXME
		tip	=> _"Choose Random Album",
		activate=> sub { my @al; for my $key (keys %::Album) { push @al,$key } my $r=int rand(@al); my $key=$al[$r]; my $ID=::FindFirstInListPlay( $::Album{$key}[::AALIST] ); ::Select(undef,undef,$ID); },
		click3	=> sub { my @list; my @al; for my $key (keys %::Album) { push @al,$key } my $nb=5; while ($nb--) { my $r=int rand(@al); push @list, splice(@al,$r,1); last unless @al; } ::PopupAA(::SONG_ALBUM,\@list,undef, ::__x( _"{album}\n<small>by</small> {artist}", album => "%a", artist => "%b"));  },
	},
	AASearch =>
	{	New	=> sub { AASearch->new($_[0]); },
	},
	SongSearch =>
	{	New	=> sub { SongSearch->new($_[0]); },
	},
	SimpleSearch =>
	{	New	=> sub { my $e=Gtk2::Entry->new; $e->signal_connect(activate => sub {my $f=$_[0]->get_text; $f=Filter->newadd(::FALSE,map $_.'s'.$f, ::SONG_TITLE,::SONG_ARTIST,::SONG_ALBUM); SetFilter($_[0]->get_toplevel,1,$f) });$e; },	#FIXME move to its own package
	},
);

our %Layouts;

use base 'Gtk2::Window';

sub new
{	my ($class,$layout,$wintype)=@_;
	$wintype||='toplevel';
	my $self=bless Gtk2::Window->new($wintype), $class;
	$self->{widgets}={};
	$self->{watcher}=::AddWatcher();
	$self->signal_connect(destroy => sub { ::RemoveWatcher($self->{watcher}); });
	::Watch($self,'Save',\&UpdateSaveSettings);
	$self->set_title(::PROGRAM_NAME);
	$self->signal_connect (show => \&show_cb);
	$self->signal_connect (window_state_event => sub
	 {	my $self=$_[0];
		my $wstate=$_[1]->new_window_state();
		warn "window $self is $wstate\n" if $::debug;
		$self->{sticky}=($wstate >= 'sticky'); #save sticky state
		0;
	 });
	$self->signal_connect(delete_event => \&close_window);
#	::set_drag($self, dest => [::DRAG_FILE,sub
#		{	my ($self,$type,@values)=@_;
#			warn "@values";
#		}],
#		motion => sub
#		{	my ($self,$context,$x,$y,$time)=@_;
#			my $target=$self->drag_dest_find_target($context, $self->drag_dest_get_target_list);
#			$context->{get_data}=1;
#			$self->drag_get_data($context, $target, $time);
#			::TRUE;
#		}
#		);
	$self->set_border_width(5);
	$self->Pack($layout);
	$self->SetFilter(0,undef);
	$self->{songlist}->FollowSong if $self->{songlist};
	$self->ChangeSelID($::SongID) if defined $::SongID;
	if ($self->{towatch})
	{	::Watch($self,'SongID',\&UpdateSongID);
		$self->UpdateSongID;
	}

	if ($wintype ne 'popup')
	{	#$self->set_position();#doesn't work before show, at least with sawfish
		$self->show_all;
		$self->move($self->{x},$self->{y}) if $self->{x};
	}
	return $self;
}

sub show_cb
{	my $self=$_[0];
	for my $key (keys %{ $self->{widgets} })
	{	my $w=$self->{widgets}{$key};
		my $sub=$w->{AfterShowAll};
		&$sub($w) if $sub;
	}
}

sub close_window
{	my $self=shift;
	$self->SaveOptions;
	unless ($self == $::MainWindow) { $self->destroy; return }
	if ($::Options{UseTray} && $::Options{CloseToTray}) { &::ShowHide; return 1}
	else { &::Quit }
}

sub UpdateSaveSettings
{	my $self=shift;
	$self->SaveOptions;
}

sub InitLayouts
{	undef %Layouts;
	my @files=$::DATADIR.::SLASH.'layouts';
	my $path=$::HomeDir.'layouts';
	if (-f $path) { push @files,$path; }
	elsif (-d $path)
	{	opendir my$d,$path;
		push @files, map $path.::SLASH.$_,sort grep m/\.layout$/, readdir $d;
		closedir $d;
	}
	ReadLayoutFile($_) for @files;
	die "No layouts file found.\n" unless keys %Layouts;
}

sub ReadLayoutFile
{	my $file=shift;
	return unless -f $file && -r $file;
	open my$fh,"<:utf8",$file;
	my $name;
	while (local $_=<$fh>)
	{	next if m/^#/;
		chomp; s/\s+$//;
		$name=undef if m/^\[/;
		if (m/^\[([^]=]+)\](?:\s*based on (.+))?$/)
		{	if (defined $2 && !exists $Layouts{$2})
			{	warn "Ignoring layout '$1' because it is based on unkown layout '$2'\n";
				next;
			}
			$name=$1;
			if (defined $2) { %{$Layouts{$name}}=%{$Layouts{$2}}; }
			else { $Layouts{$name}=undef; }
			next;
		}
		s/_\"([^"]+)"/_($1)/ge;	#translation, escaping the " so it is not picked up as a translatable string
		next unless defined $name && m/^(\w+)\s*=\s*(.*)$/;
		if ($2 eq '') {delete $Layouts{$name}{$1};next}
		$Layouts{$name}{$1}= $2;
	}
	close $fh;
}

sub SaveOptions
{	my $self=shift;
	#warn "saving ".$self->{layout}." window options\n";
	my @data=(join ',',$self->get_position,$self->get_size,$self->{sticky});
	@data=(join ',','','',$self->get_size) if $self->{donotsavepos};
	for my $key (keys %{ $self->{widgets} })
	{	my $w=$self->{widgets}{$key};
		my $sub=$w->{SaveOptions};
		next unless $sub;
		$sub=&$sub($w) if ref $sub;
		push @data,$key,$sub if defined $sub;
		#warn "@data";
	}
	$::Options{'Layout_'.$self->{layout}}=join ' ',@data;
}

sub Pack
{	my ($self,$layout)=@_;
	unless (exists $Layouts{$layout})
	{	warn "Layout $layout not found, reverting to default\n";
		$layout='default player layout'; #FIXME if not a player window
	}
	$self->{layout}=$layout;
	my $options= $::Options{'Layout_'.$self->{layout}} || '-1,-1,'.$Layouts{$layout}{Default};
	my ($wopt,%opt)=split / /,$options;
	my ($x,$y,$w,$h,$sticky)=split /,/,$wopt;
	$self->resize($w,$h) if $w;
	$self->stick if $sticky;
	if (my $scrn=Gtk2::Gdk::Screen->get_default)
	 {$x=-1 if $x>$scrn->get_width || $y>$scrn->get_height}
	if ($x>-1 && $y>-1)
	{ $self->move($x,$y);
	  $self->{x}=$x; $self->{y}=$y;	#move before show_all rarely works, so save pos to set it later
	}

	my $boxes=$Layouts{$layout};
	$self->{KeyBindings}=::make_keybindingshash($boxes->{KeyBindings}) if $boxes->{KeyBindings};
	$self->{towatch}={};
	my $widgets=$self->{widgets};

	# create boxes
	my @boxlist;
	for my $key (keys %$boxes)
	{	my $type=substr $key,0,2;
		$type=$Player::Boxes::Boxes{$type};
		next unless $type;
		$widgets->{$key}= &{ $type->{New} }( $opt{$key} );
		push @boxlist,$key;
	}
	#pack boxes
	for my $key (@boxlist)
	{	my $type=substr $key,0,2;
		$type=$Player::Boxes::Boxes{$type};
		my $box=$widgets->{$key};
		my $line=$boxes->{$key};
		my @names= $type->{Parse}?	&{ $type->{Parse} }( $box, $line )
					 :	$line=~m/$::re_spaces_unlessinbrackets/g;
		for my $name (@names)
		{	my $extra;
			$extra=$1 if $name=~s/\(([^\)]*)\)$//; #remove (...) and put it in $extra
			my $wg=$widgets->{$name} ||= $self->NewObject($name,$extra,$opt{$name});
			next unless $wg;
			if ($wg->parent) {warn "layout error: $name already has a parent -> can't put it in $key\n"; next;}

			$self->{songlist}=$wg if $wg->isa('SongList');
			&{ $type->{Pack} }( $box,$wg );
		}
	}

	for my $key (grep m/^[HV]Size/, keys %$boxes)
	{	my $mode= ($key=~m/^V/)? 'vertical' : 'horizontal';
		my @names=split / /,$boxes->{$key};
		if ( $names[0]=~m/^\d+$/ )
		{	my $s=shift @names;
			my @req=($mode eq 'vertical')? (-1,$s) : ($s,-1);
			$widgets->{$names[0]}->set_size_request(@req);
			next if @names==1;
		}
		my $sizegroup=Gtk2::SizeGroup->new($mode);
		$sizegroup->add_widget( $widgets->{$_} ) for @names;
	}
	if (my $l=$boxes->{VolumeScroll})
		{ $widgets->{$_}->signal_connect(scroll_event => \&::ChangeVol) for split / /,$l; }
	$self->signal_connect_after(key_press_event => \&KeyPressed);

	my @noparentboxes=grep m/^(?:[HVM][BP]|[ET]B)/ && !$widgets->{$_}->parent, keys %$boxes;
	if (@noparentboxes!=1) {warn "layout error: (@noparentboxes) have no parent -> can't find toplevel box\n"}
	$self->add( $widgets->{ $noparentboxes[0] } );

	my $title=$boxes->{Title} || _"%S by %a";
	if (my @l=::UsedFields($title)) { $self->{TitleString}=$title; $self->{towatch}{$_}=undef for @l; }
	else { $self->set_title($title) }
}

sub NewObject
{	my ($self,$name,@extra)=@_;
	$name=~s/\d+$//;
	my $ref=$objects{$name};
	unless ($ref) { warn "layout error : unknown object $name\n"; return undef; }
	#warn $name;
	$extra[1]=$ref->{default} if !defined $extra[1];
	my $widget=($ref->{class})
		? $ref->{class}->new($ref,@extra)
		: &{ $ref->{New} }(@extra);
	$widget->{'ref'}=$ref;
	if (grep m/^click/, keys %$ref)
	{	$widget->signal_connect(button_press_event => \&Button_press_cb);
	}
	if ( $ref->{tip} && !ref $ref->{tip} )
	{	$::Tooltips->set_tip( $widget, $ref->{tip} );
	}
	if (my $f=$ref->{fields})
	{	my @f=(ref $f)? @$f : $f;
		$self->{towatch}{$_}=undef for @f;
	}
	if ($ref->{markup})
	{	$self->{towatch}{$_}=undef for ::UsedFields( $ref->{markup} );
	}
	if ($ref->{event})
	{	my $sub=$ref->{update} || \&UpdateObject;
		my @e=($ref->{event});
		@e=@{$e[0]} if ref $e[0];
		::Watch($widget,$_,$sub ) for @e;
		&$sub($widget) unless $ref->{noinit};
	}
	::set_drag($widget,source => $ref->{dragsrc}, dest => $ref->{dragdest});
	return $widget;
}

sub UpdateObject
{	my $widget=$_[0];
	my $ref=$widget->{'ref'};
	if ( my $tip=$ref->{tip} )
	{	$tip=&$tip if ref $tip;
		$::Tooltips->set_tip($widget,$tip);
	}
	if ( my $pic=$ref->{pic} )
	{	$pic=&$pic if ref $pic eq 'CODE';
		return if $widget->{pic} eq $pic;
		$widget->{pic}=$pic;
		$pic=$pic->[0] if ref $pic;#warn $pic;
		Setpic($widget,$pic,$ref->{size});
	}
}
sub Setpic
{	my ($wdgt,$stockid,$size)=@_;
	$wdgt->child->set_from_stock($stockid,$size);
}
sub Enter_Callback
{	my $ref=$_[0]->{pic};
	return unless ref $ref && $ref->[1];
	Setpic($_[0],$ref->[1],$_[0]->{size});
	0;
}
sub Leave_Callback
{	my $ref=$_[0]->{pic};
	return unless ref $ref && $ref->[0];
	Setpic($_[0],$ref->[0],$_[0]->{size});
	0;
}

sub Button_press_cb
{	(my $self,$::LEvent)=@_;
	my $ref=$self->{'ref'};
	$::LEventW=$self->get_toplevel;
	my $key='click'.$::LEvent->button;
	my $sub=$ref->{$key};
	return 0 if !$sub && $ref->{activate};
	$sub||=$ref->{click1};
	return 0 unless $sub;
	if (ref $sub)	{&$sub}
	else		{ ::run_command($self,$sub) }
	1;
}

sub UpdateSongID
{	my $self=shift;
	my $ID=$::SongID;
	if (defined $ID)
	{ ::ChangeWatcher($self->{watcher},[$ID],[keys %{$self->{towatch}}],sub { $self->UpdateSongID } );
	}
	else
	{ ::ChangeWatcher($self->{watcher});
	}

	if (my $title=$self->{TitleString})
	{	$title= defined $ID	? ::ReplaceFields($ID,$title)
					: '<'._("Playlist Empty").'>';
		$self->set_title($title);
	}

	for my $w (values %{ $self->{widgets} })
	{	next unless $w->{'ref'};
		my $ref=$w->{'ref'};
		&{ $ref->{schange} }($w,$ID) if $ref->{schange};
		if ($ref->{markup})
		{	my $l=$w->{label};
			if (defined $ID) { $l->set_markup(::ReplaceFieldsAndEsc( $ID,$ref->{markup} )); }
			elsif ($ref->{markup_empty}) { $l->set_markup($ref->{markup_empty}); }
			else { $l->set_markup(''); }
		}
	}
	#$self->resize(1,1);
#	$self->UpdateTime;
}

sub ChangeSelID
{	my ($self,$ID)=@_;
	$self->{SelID}=$ID;
	for my $key (keys %{ $self->{widgets} })
	{	my $w=$self->{widgets}{$key};
		my $sub=$w->{OnChangedID};
		&$sub($w,$ID) if $sub;
	}
}
sub ChangedSelection
{	my $self=shift;
	for my $key (keys %{ $self->{widgets} })
	{	my $w=$self->{widgets}{$key};
		my $sub=$w->{OnChangedSelection};
		&$sub($w) if $sub;
	}
}
#sub SetSort
#{	my($self,$sort)=@_;
#	$self->{songlist}->Sort($sort);
#}
sub SetFilter
{	my ($self,$nb,$filter)=@_;
	$filter=Filter->new($filter) unless defined $filter && ref $filter eq 'Filter';
	$self->{'Filter'.$nb}=$filter;
	$self->{'Filter'.$_}=undef for ($nb+1)..9;
	$self->{Filter}=Filter->newadd(TRUE, map($self->{'Filter'.$_},0..9) );

	$self->UpdateFilter;
}
sub UpdateFilter
{	my $self=shift;
	for my $key (keys %{ $self->{widgets} })
	{	my $w=$self->{widgets}{$key};
		my $sub=$w->{OnChangedFilter};
		#warn "$key\n" if $sub;
		&$sub($w) if $sub;
	}
}

sub Vshrink
{	my $self=shift;
	my ($w,$h)=$self->get_size;
	$self->resize($w,1);
	$self->{height}=$h;
}
sub Vgrow
{	my $self=shift;
	return unless $self->{height};
	my ($w,$h)=$self->get_size;
	$self->resize($w,delete $self->{height});
}

sub KeyPressed
{	my ($self,$event)=@_;
	my $keyval=$event->keyval;
	my $mod;
	$mod.='c' if $event->state >= 'control-mask';
	$mod.='a' if $event->state >= 'mod1-mask';
	$mod.='s' if $event->state >= 'shift-mask';
	$keyval=$mod.'-'.$keyval if $mod;
	my ($cmd,$arg);
	if ( exists $::CustomBoundKeys{$keyval} )
	{	$cmd= $::CustomBoundKeys{$keyval};
	}
	elsif ($self->{KeyBindings} && exists $self->{KeyBindings}{$keyval} )
	{	$cmd= $self->{KeyBindings}{$keyval};
	}
	if ( exists $::GlobalBoundKeys{$keyval} )
	{	$cmd= $::GlobalBoundKeys{$keyval};
	}
	return 0 unless $cmd;
	::run_command($self,$cmd);
	return 1;
}

sub EnqueueSelected
{	my $self=shift;
	return unless $self;
	$self=$self->get_toplevel;
	return unless $self->{songlist};
	$self->{songlist}->EnqueueSelected;
}

#################################################################################

sub RemoveFilter
{	::Select('') if defined $::ListMode || !$::SelectedFilter->is_empty;
}

sub SortMenu
{	my $return=0;
	$return=1 unless @_;
	my $check=$::Options{Sort};
	my $found;
	my $callback=sub { ::Select(undef,$_[1]); };
	my $append=sub
	 {	my ($menu,$name,$sort,$true,$cb)=@_;
		$cb||=$callback;
		$true=($sort eq $check) unless defined $true;
		my $item = Gtk2::CheckMenuItem->new($name);
		$item->set_draw_as_radio(1);
		$item->set_active($found=1) if $true;
		$item->signal_connect (activate => $cb, $sort );
		$menu->append($item);
	 };
	my $menu = Gtk2::Menu->new;

	my $submenu= Gtk2::Menu->new;
	my $sitem = Gtk2::MenuItem->new(_"Weighted Random");
	for my $name (sort keys %::SavedWRandoms)
	{	&$append($submenu,$name, $::SavedWRandoms{$name} );
	}
	my $editcheck=(!$found && $check=~m/^r/);
	&$append($submenu,_"Edit ...", undef, $editcheck, sub
		{	#$::LEventW=$window;
			::EditWeightedRandom($::Options{Sort},undef,\&::Select_sort);
		});
	$sitem->set_submenu($submenu);
	$menu->prepend($sitem);

	&$append($menu,_"Shuffle",'s',undef,\&::Shuffle);

	{ my $item=Gtk2::CheckMenuItem->new(_"Repeat");
	  $item->set_active($::Options{Repeat});
	  $item->set_sensitive(0) if $::RandomMode;
	  $item->signal_connect(activate => sub {$::Options{Repeat}=$_[0]->get_active ? 1 : 0;} );
	  $menu->append($item);
	}

	$menu->append(Gtk2::SeparatorMenuItem->new); #separator between random and non-random modes

	&$append($menu,_"list order", '' ) if defined $::ListMode;
	for my $name (sort keys %::SavedSorts)
	{	&$append($menu,$name, $::SavedSorts{$name} );
	}
	&$append($menu,_"Edit...",undef,!$found,sub
		{	#$::LEventW=$window;
			::EditSortOrder($::Options{Sort},undef,\&::Select_sort);
		});
	$menu->show_all;
	return $menu if $return;
	$menu->popup(undef,undef,\&::menupos,undef,$::LEvent->button,$::LEvent->time);
}

sub FilterMenu
{	my $return=0;
	$return=1 unless @_;
	my $check=$::SelectedFilter->{string};
	my $menu = Gtk2::Menu->new;
	my $item_callback=sub { ::Select($_[1]); };
	for my $list (sort keys %::SavedFilters)
	{	next if $list eq 'Playlist';
		my $filt=$::SavedFilters{$list}->{string};
		my $text=$list; $text=~s/^_//;
		my $item = Gtk2::CheckMenuItem->new($text);
		$item->set_draw_as_radio(1);
		$item->set_active(1) if $filt eq $check;
		$item->signal_connect ( activate =>  $item_callback ,$filt );
		if ($list eq 'Library') {$menu->prepend($item);}
		else			{$menu->append($item);}
	}
	my $item=Gtk2::CheckMenuItem->new(_"Edit...");
	$item->set_draw_as_radio(1);
	$item->signal_connect ( activate => sub
		{ ::EditFilter($::LEventW,$::SelectedFilter,undef,\&::Select);
		});
	$menu->append($item);
	if (keys %::SavedLists)
	{	my $submenu=Gtk2::Menu->new;
		my $list_cb=sub { ::Select( undef,'',undef,undef,$_[1] ) };
		for my $list (keys %::SavedLists)
		{	my $item = Gtk2::CheckMenuItem->new($list);
			$item->set_draw_as_radio(1);
			$item->set_active(1) if defined $::ListMode && $list eq $::ListMode;
			$item->signal_connect( activate =>  $list_cb, $list );
			$submenu->append($item);
		}
		my $sitem=Gtk2::MenuItem->new(_"Saved Lists");
		#my $sitem=Gtk2::CheckMenuItem->new('Saved Lists');
		#$item->set_draw_as_radio(1);
		$sitem->set_submenu($submenu);
		$menu->prepend($sitem);
	}
	$menu->show_all;
	return $menu if $return;
	$menu->popup(undef,undef,\&::menupos,undef,$::LEvent->button,$::LEvent->time);
}

sub NewMenuBar
{	my $self=Gtk2::MenuBar->new;
	$self->signal_connect( button_press_event => sub
	 {	return 0 if $_[0]->{busy};
		$_[0]->{busy}=1;
		for my $item ($_[0]->get_children)
		{	my $ref=$item->{'ref'};
			next unless $ref->{setmenu};
			my $submenu=&{$ref->{setmenu}}($item);
			$item->set_submenu($submenu);
			$submenu->show_all;
		}
		0;
	 });
	$self->signal_connect( selection_done => sub { $_[0]->{busy}=undef; });

	return $self;
}

package Player::Boxes;
use strict;
use warnings;
use Gtk2;

our %Boxes=
(	HB	=>
	{	New	=> sub { Gtk2::HBox->new(::FALSE,0); },
		Parse	=> \&SBoxParse,
		Pack	=> \&SBoxPack,
	},
	VB	=>
	{	New	=> sub { Gtk2::VBox->new(::FALSE,0); },
		Parse	=> \&SBoxParse,
		Pack	=> \&SBoxPack,
	},
	HP	=>
	{	New	=> sub { PanedNew('Gtk2::HPaned',$_[0]); },
		Parse	=> \&PanedParse,
		Pack	=> \&PanedPack,
	},
	VP	=>
	{	New	=> sub { PanedNew('Gtk2::VPaned',$_[0]); },
		Parse	=> \&PanedParse,
		Pack	=> \&PanedPack,
	},
	TB	=>
	{	New	=> \&NewTB,
		Parse	=> \&ParseTB,
		Pack	=> \&PackTB,
	},
	MB	=>
	{	New	=> \&Player::NewMenuBar,
		Pack	=> sub { $_[0]->append($_[1]); },
	},
	EB	=>
	{	New	=> sub { Gtk2::Expander->new; },
		Pack	=> sub { $_[0]->add($_[1]); },
	},
);

sub NewTB
{	my $opt=$_[0];
	my $nb=Gtk2::Notebook->new;
	if ($opt && $opt=~m/page=(\d+)/) { $nb->{SetPage}=$1; }
	$nb->{SaveOptions}=sub { 'page='.$_[0]->get_current_page };
	return $nb;
}
sub ParseTB
{	my ($box,$line)=@_;
	my @words= $line=~m/((?:"[^"]*[^\\]")|[^ ]*) $::re_spaces_unlessinbrackets/og; #format : "title with spaces" widget1(options with spaces) ...
	my @names;
	while (@words>1)
	{	push @{$box->{Titles}},shift @words;
		push @names,shift @words;
	}
	s/^"// && s/"$// for @{$box->{Titles}};
	return @names;
}
sub PackTB
{	my ($nb,$wg)=@_;
	$nb->append_page($wg, Gtk2::Label->new( shift @{$nb->{Titles}} ) );
	my $n=$nb->{SetPage};
	if (defined $n && $n==($nb->get_n_pages-1)) {$wg->show; $nb->set_current_page($n);}
}

sub SBoxParse
{	my ($box,$line)=@_;
	my @names;
	for my $name ($line=~m/$::re_spaces_unlessinbrackets/g)
	{	$name=~s/^([0-9]+)?(_)?(-)?//;
		my $padding= $1 || 0;
		my $expand=$2? 1 : 0;
		my $end=   $3? 1 : 0;
		push @{ $box->{PackOptions} }, $padding,$expand,$end;
		push @names,$name;
	}
	return @names;
}
sub SBoxPack
{	my ($box,$wg)=@_;
	my ($pad,$exp,$end)= splice @{ $box->{PackOptions} },0,3;
	if ($end)	{ $box->pack_end(   $wg,$exp,::TRUE,$pad ); }
	else		{ $box->pack_start( $wg,$exp,::TRUE,$pad ); }
}

sub PanedParse
{	my ($box,$line)=@_;
	my @names= $line=~m/$::re_spaces_unlessinbrackets/g;
	$box->{PackOptions}= [ map s/^_//, @names ];
	return @names;
}
sub PanedPack
{	my ($paned,$wg,$opt)=@_;
	my $expand= shift @{ $paned->{PackOptions} };
	if (!$paned->child1)	{$paned->pack1($wg,$expand,::TRUE);}
	elsif (!$paned->child2)	{$paned->pack2($wg,$expand,::TRUE);}
	else {warn "layout error : trying to pack more than 2 widgets in a paned container\n"}
}

sub PanedNew
{	my ($class,$opt)=($_[0],$_[1]);
	my $pane=$class->new;
	($pane->{pos1},$pane->{pos2})=split /_/,$opt if $opt;
	$pane->set_position($pane->{pos1}) if defined $pane->{pos1};
	$pane->{SaveOptions}=sub { $_[0]->{pos1}.'_'.$_[0]->{pos2} };
	$pane->signal_connect(size_allocate => \&Paned_size_cb ); #needed to correctly behave when a child is hidden
	$pane;
}

sub Paned_size_cb
{	my ($self,$alloc)=@_;
	$alloc=$self->isa('Gtk2::VPaned')? $alloc->height : $alloc->width;
	if (defined $self->{pos1} && defined $self->{pos2} && $alloc != ($self->{pos1} + $self->{pos2}))
	{	if    ($self->child1_resize && !$self->child2_resize)	{ $self->{pos1}=$alloc-$self->{pos2}; }
		elsif ($self->child2_resize && !$self->child1_resize)	{ $self->{pos2}=$alloc-$self->{pos1}; }
		else {return}
		$self->set_position( $self->{pos1} );
	}
	else { $self->{pos1}=$self->get_position; $self->{pos2}=$alloc-$self->{pos1}; }
}

package Player::Button;
use strict;
use warnings;
use Gtk2;

use base 'Gtk2::Button';

sub new
{	my ($class,$ref,$extra)=@_;
	my $self = bless Gtk2::Button->new, $class;
	$self->{extra}=$extra;
	$self->set_relief('none');
	if ($ref->{pic})
	{	$self->add(Gtk2::Image->new);
		my $picref=$ref->{pic};
		$picref=&$picref if ref $picref eq 'CODE';
		Player::Setpic($self, $picref->[0], $ref->{size} );
		if ( @$picref >1 )
		{ $self->{size}=$ref->{size};
		  $self->signal_connect(enter_notify_event => \&Player::Enter_Callback);
		  $self->signal_connect(leave_notify_event => \&Player::Leave_Callback);
		}
	}
	else { $self->set_label($ref->{label}); }
	$self->signal_connect( clicked =>  $ref->{activate} ) if $ref->{activate};
	return $self;
}

package Player::Flag;
use strict;
use warnings;
use Gtk2;

use base 'Gtk2::EventBox';

sub new
{	my ($class,$ref)=@_;
	my $self = bless Gtk2::EventBox->new, $class;
	$self->add(Gtk2::Image->new);
	$self->{size}=$ref->{size};
	$self->signal_connect(enter_notify_event => \&Player::Enter_Callback);
	$self->signal_connect(leave_notify_event => \&Player::Leave_Callback);
	return $self;
}

package Player::Label;
use strict;
use warnings;
use Gtk2;

use base 'Gtk2::EventBox';

sub new
{	my ($class,$ref)=@_;
	my $self = bless Gtk2::EventBox->new, $class;
	my $label=$self->{label}=Gtk2::Label->new;
	if ($ref->{font}) { $label->modify_font(Gtk2::Pango::FontDescription->from_string($ref->{font})); }
	$label->set_alignment(0,.5);
	$self->add($label);
	return $self;
}

package Player::ScrollingLabel;
use strict;
use warnings;
use Gtk2;

use constant
{	SPEED	=> 100,
	INCR	=> 5,
};

use base 'Gtk2::Viewport';

my ($Scroll,%Scroll,%Adj);

sub new
{	my ($class,$ref,$extra)=@_;
	my $self = bless Gtk2::Viewport->new, $class;
	$self->set_shadow_type('none');
	my $label=$self->{label}=Gtk2::Label->new;
	$label->set_alignment(0,.5);
	my $eventbox=Gtk2::EventBox->new;
	$eventbox->add($label);
	$self->add($eventbox);
#$eventbox->signal_connect(enter_notify_event => sub {$_[0]->child->set_markup('<u>'.$_[0]->child->get_label.'</u>')});
#$eventbox->signal_connect(leave_notify_event => sub {my $m=$_[0]->child->get_label; $m=~s#^<u>##;$m=~s#</u>$##; $_[0]->child->set_markup($m)});
	my $size=$extra||$ref->{size};
	($size)=$label->create_pango_layout('X' x $size)->get_pixel_size;
	$self->set_size_request($size,-1);
	my $id=1;
	$id++ while exists $Adj{$id};
	$Adj{$id}=$self->get_hadjustment;
	$self->signal_connect(destroy => sub { delete $Scroll{$id}; delete $Adj{$id}; });
	$eventbox->signal_connect(enter_notify_event => sub
		{  $Scroll{ $_[2] }=INCR;
		   $Scroll||=Glib::Timeout->add(SPEED, \&Scroll);
		},$id);
	$eventbox->signal_connect(leave_notify_event => sub
		{  $Scroll{ $_[2] }=-INCR();
		   $Scroll||=Glib::Timeout->add(SPEED, \&Scroll);
		},$id);

	return $self;
}

sub Scroll	#called by a TimeOut, scroll all instances of ScrollingLabel
{	unless (keys %Scroll) {return $Scroll=0}
	while (my ($id,$i)=each %Scroll)
	{	my $adj=$Adj{$id};
		my $max=$adj->upper - $adj->page_size;
		my $x=$adj->get_value+$i;
		if ($max>0)
		{	if    ($x>$max) {$x=$max}
			elsif ($x<0)	{$x=0}
			$adj->set_value($x);
		}
		#remove scroll if reach an end or if label fit
		delete $Scroll{$id} if $x>=$max || $x==0;
	}
	1;
}

package Player::TimeSlider;
use strict;
use warnings;
use Gtk2;

use base 'Gtk2::HScale';

sub new
{	my ($class,$ref)=@_;
	my $adj=Gtk2::Adjustment->new (0, 0, 300, 1, 10, 1);
	my $self = bless Gtk2::HScale->new($adj), $class;
	$self->set_draw_value(::FALSE);
	$self->{adj}=$adj;
	$self->signal_connect(button_press_event => \&button_press_cb);
	$self->signal_connect(button_release_event => \&button_release_cb);
	return $self;
}

sub button_press_cb
{	my $self=$_[0];
	$self->{pressed}= $self->{adj}->signal_connect(value_changed  => \&value_changed_cb,$self);
	my $label=$self->get_toplevel->{widgets}{LabelTime};
	$label->{busy}=1 if $label;
	0;
}

sub button_release_cb
{	my $self=$_[0];
	return unless $self->{pressed};
	$self->{adj}->signal_handler_disconnect( $self->{pressed} );
	$self->{pressed}=0;
	my $label=$self->get_toplevel->{widgets}{LabelTime};
	$label->{busy}=0 if $label;
	::SkipTo( $self->{adj}->get_value );
	0;
}

sub value_changed_cb
{	my ($adj,$self)=@_;
	my $label=$self->get_toplevel->{widgets}{LabelTime};
	return unless $label;
	my $s=$adj->get_value;
	my $format=($::Songs[$::SongID][::SONG_LENGTH]<600)? '%01d:%02d' : '%02d:%02d';
	$label->set_label( sprintf $format,int($s/60),$s%60 );
}

package Player::Cover;
use strict;
use warnings;
use Gtk2;

use base 'Gtk2::EventBox';

sub new
{	my ($class,$ref,$extra)=@_;
	my $self = bless Gtk2::EventBox->new, $class;
	$self->{key}='';
	#$minsize||=$ref->{size};
	my $img=Gtk2::Image->new;
	#$img->{size}=0;
	$img->{maxsize}=$extra || 200;
	$img->signal_connect(size_allocate => \&size_allocate_cb);
	$img->signal_connect(destroy => sub {delete $::ToDo{'8_LoadImg'.$_[0]}});
	$self->add($img);
	#$self->set_size_request($minsize,$minsize);
	return $self;
}

sub AAPicture_Changed
{	my ($self,$key)=@_;
	return if $key ne $self->{key};
	$self->{key}='';
	$self->set($key);
}

sub set
{	my ($self,$key)=@_;
	return if $self->{key} eq $key;
	$self->{key}=$key;
	my $file=(defined $key)? $::Album{ $key }[::AAPIXLIST] : undef;
	#my $img=$self->child;
	#::ScaleImageFromFile( $img, $img->{size}, $file );
	#if ($img->{pixbuf})	{ $self->show; }
	#else			{ $self->hide; }
	if ($file)
	{my $img=$self->child;
	 $img->{pixbuf}=undef;
	 $img->set_from_pixbuf(undef);
	 $self->show;
	 ::IdleDo('8_LoadImg'.$img,500,sub
	 {	my ($self,$file)=@_;
		my $img=$self->child;
		::ScaleImageFromFile( $img, $img->{size}, $file );
		$self->hide unless $img->{pixbuf};
	 },$self,$file);
	}
	else
	{ $self->hide;
	}
}

sub size_allocate_cb
{	my ($img,$alloc)=@_;
	my $max=$img->{maxsize};
	my $h=$alloc->height;  my $w=$alloc->width;
	my $s=($h<$w)? $h : $w; $s=$max if $s>$max;
	if (!defined $img->{size})
	{	return unless $img->{pixbuf} || $::ToDo{'8_LoadImg'.$img};
		$h=$max if $h>$max; $s=$h; $img->set_size_request($h,$h);
	}
	elsif ($img->{size}==$s) {return}
	$img->{size}=$s;
	::ScaleImage( $img, $s ) unless $::ToDo{'8_LoadImg'.$img};
}

package Player::TogButton;
use strict;
use warnings;
use Gtk2;

use base 'Gtk2::ToggleButton';
sub new
{	my ($class,$ref,$opt1,$opt2)=@_;
	my $self = bless Gtk2::ToggleButton->new, $class;
	my %opt1;
	if ($opt1)	{ $opt1{$1}=$2 while $opt1=~m/(\w+)=([^,]*)(?:,|$)/g; }
	my $label=	$opt1{label}	? Gtk2::Label->new($opt1{label}) :
			$opt1{icon}	? Gtk2::Image->new_from_stock($opt1{icon},'menu') : undef;
	$self->add( $label );
	#$self->{gravity}=$opt1{gravity};
	$self->{SaveOptions}=\&SaveOptions;
	if ($opt2 && $opt2=~m/state=0/) { $self->{AfterShowAll}=sub { $_[0]->toggled_cb($opt1{widget}) }; }
	else { $self->set_active(1); }
	$self->signal_connect( toggled => \&toggled_cb, $opt1{widget} );

	return $self;
}

sub SaveOptions
{	'state='. ( $_[0]->get_active ? '1' : '0' );
}

sub toggled_cb
{	my ($self,$widget)=@_;
	my @w=map $self->get_toplevel->{widgets}{$_}, split /\|/,$widget;
	#my $grav=$self->get_toplevel->get_gravity;
	#$self->get_toplevel->set_gravity($self->{gravity}) if $self->{gravity};
	if ($self->get_active) { $_->show for @w }
	else { $_->hide for @w }
	#$self->get_toplevel->set_gravity($grav);
}

package Player::Context;
use strict;
use warnings;
use Gtk2;

use base 'Gtk2::VBox';

sub new
{	my ($class,$ref,$opt1,$opt2)=@_;
	my $self = bless Gtk2::VBox->new, $class;
	$self->{notebook}=Gtk2::Notebook->new;

	$self->{followselection}=1 if $opt1 && $opt1 eq 'followselection';
	my $check= $self->{followselection} ? _"Follow selected song" : _"Follow playing song";
	$check=Gtk2::CheckButton->new($check);
	$self->{follow}= ($opt2 && $opt2=~m/follow=1/)? 1 : 0;
	$check->set_active( $self->{follow} );
	$check->signal_connect(toggled => sub { $self->{follow}=$_[0]->get_active; $self->SongChanged($self->GetID) if $self->{follow}; });
	::set_drag($check, dest => [::DRAG_ID,sub
		{	my ($check,$type,@IDs)=@_;
			$self->SongChanged($IDs[0],1);
		}]);

	$self->pack_start($check,::FALSE,::FALSE,1);
	$self->add( $self->{notebook} );
	for my $c (keys %::Context)
	{	$self->Append($c);
	}
	::Watch($self,'SongID',\&PlayingSongChanged);
	::Watch($self,'Context',\&Changed);
	$self->{OnChangedID}=\&SelSongChanged;
	$self->{SaveOptions}=\&SaveOptions;

	$self->{ID}=$::SongID;
	#$self->SongChanged($::SongID);

	return $self;
}

sub SaveOptions
{	my $self=$_[0];
	return 'follow='.$self->{follow};
}

sub Changed
{	my ($self,$action,$key)=@_;
	my $notebook=$self->{notebook};
	if ($action eq 'add') { $self->Append($key); }
	elsif ($action eq 'remove')
	{	for my $w ($notebook->get_children)
		{	if ($w->{context_key} eq $key)
			 { $notebook->remove_page($notebook->page_num($w)); $w->destroy; }
		}
	}
}

sub GetID
{	$_[0]->{followselection} ? $_[0]->get_toplevel->{SelID} : $::SongID;
}

sub PlayingSongChanged
{	$_[0]->SongChanged($::SongID) unless $_[0]->{followselection};
}

sub SelSongChanged
{	&SongChanged if $_[0]->{followselection};
}

sub SongChanged
{	my ($self,$ID,$force)=@_;
	return unless $self->{follow} || $force;
	$self->{ID}=$ID;
	my $nb=$self->{notebook};
	my $page=$nb->get_nth_page( $nb->get_current_page );
	return unless $page;
	$page->SongChanged($ID) if $page->mapped;
	#for my $w ($self->{notebook}->get_children)
	#{	$w->SongChanged($ID);
	#}
}

sub Append
{	my ($self,$key)=@_;
	my @widgets=$::Context{$key}->new;
	for my $w (@widgets)
	{	$w->{context_key}=$key;
		$w->signal_connect(map => sub { $_[0]->SongChanged($self->{ID}) });
		$self->{notebook}->append_page($w, Gtk2::Label->new($w->title) );
		#$w->SongChanged( $self->GetID );
	}
}

1;
