package GMB::Plugin::FetchCover;
use strict;
use warnings;
require 'simple_http.pm';
use base 'Gtk2::Window';
use constant
{	PID => 'FETCHCOVER',#used to identify the plugin and its options
};
my %menuentry=
(	label => _"Look for a picture in google images",	#label of the menu entry
	code => sub {Fetch($_[0]{ID})},			#when menu entry selected
	test => sub {$_[0]{col}==::SONG_ALBUM},		#the menu entry is displayed if returns true
);

::SetDefaultOptions(PID, USEFILE => 1, COVERFILE => 'cover');

$::Plugins{&PID}=__PACKAGE__;

sub init
{	push @::cMenuAA,\%menuentry;
}
sub end
{	@::cMenuAA =grep $_!=\%menuentry, @::SongCMenu;
}
sub name { _"Fetch cover" }

sub prefbox
{	my $label1=Gtk2::Label->new(_"FetchCover plugin");
	my $label2=Gtk2::Label->new(_"Adds a menu entry to album context menu, allowing to search the cover in google and save it.");
	$label2->set_line_wrap(::TRUE);
	my $check1=::NewPrefCheckButton('PLUGIN_'.PID.'_ASK',_"Ask confirmation only if file already exists");
	my $check2=::NewPrefCheckButton('PLUGIN_'.PID.'_UNIQUE',_"Find a unique filename if file already exists");
	my $entry1=::NewPrefEntry('PLUGIN_'.PID.'_COVERPATH','');
	my $entry2=::NewPrefEntry('PLUGIN_'.PID.'_COVERFILE','');
	my ($radio1a,$radio1b)=::NewPrefRadio('PLUGIN_'.PID.'_USEPATH',undef,_"use song folder",0,_"use :",1);
	my ($radio2a,$radio2b)=::NewPrefRadio('PLUGIN_'.PID.'_USEFILE',undef,_"use album name",0,_"use :",1);
	my $frame1=Gtk2::Frame->new(_"default folder");
	my $frame2=Gtk2::Frame->new(_"default filename");
	my $vbox1=::Vpack( $radio1a,[$radio1b,$entry1] );
	my $vbox2=::Vpack( $radio2a,[$radio2b,$entry2] );
	$frame1->add($vbox1);
	$frame2->add($vbox2);
	return ::Vpack( $label1,$label2,$frame1,$frame2,$check1,$check2 );
}

sub Fetch
{	my $ID=$_[0];
	my $album=$::Songs[$ID][::SONG_ALBUM];
	my $artist=$::Songs[$ID][::SONG_ARTIST];
	$artist='' if $artist eq '<Unknown>';
	$album='' if $album=~m/^<Unknown>/;

	my $self=bless Gtk2::Window->new;
	$self->set_border_width(4);
	$self->set_default_size(300,100);
	my $vbox=Gtk2::VBox->new;
	my $hbox=Gtk2::HBox->new;
	$self->{searchentry}=my $entry=Gtk2::Entry->new;
	my $Bsearch=::NewIconButton('gtk-find','search');
	$self->{Bstop}=my $Bstop=::NewIconButton('gtk-stop','stop');
	$self->{progress}=Gtk2::ProgressBar->new;
	$self->{table}=my $table=Gtk2::Table->new(4,5,::TRUE);
	$hbox->add($entry);
	$hbox->pack_start($_,::FALSE,::FALSE,2) for $Bsearch,$Bstop;
	$vbox->pack_start($hbox,::FALSE,::FALSE,2);
	$vbox->add($table);
	$vbox->pack_start($self->{progress},::FALSE,::FALSE,2);
	$self->add($vbox);
	$self->show_all;
	$table->{album}=$album;
	$table->{dir}=$::Songs[$ID][::SONG_PATH];
	my $search=$album=~m/^<Unknown>/ ? '' : "\"$album\"";
	$search.=" \"$artist\"" unless $search eq '' || $artist eq '<Unknown>';
	$entry->set_text($search);
	$search=~s/([^A-Za-z0-9-])/sprintf('%%%02X', ord($1))/seg;

	$self->signal_connect(destroy => sub {delete $::ToDo{'8_FetchCovers'.$self};$self->{searchid}++});
	$entry->signal_connect( activate => \&search_cb );
	$Bsearch->signal_connect( clicked => \&search_cb );
	$Bstop->signal_connect( clicked => sub { $self->stop });
	$self->search_cb unless $entry->get_text eq '';
}

sub search_cb
{	my $self=$_[0];
	$self=::find_ancestor($self,__PACKAGE__);
	if ($self->{waiting}) {$self->{waiting}->abort;}
	$self->{Bstop}->set_sensitive(1);
	my $entry=$self->{searchentry};
	my $table=$self->{table};
	my $s=$entry->get_text;
	$table->remove($_) for $table->get_children;
	$table->{x}=$table->{y}=0;
	$self->{toload}=[];
	$s=~s/([^A-Za-z0-9-])/sprintf('%%%02X', ord($1))/seg;
	$self->{waiting}=Simple_http::get_with_cb(cb => sub {$self->searchresults_cb(@_)}, url => 'http://images.google.com/images?q='.$s.'&imgsz=medium|large', cache=>1);
	#$self->stop unless $self->{waiting};
}

sub searchresults_cb
{	my ($self,$result)=@_;
	$self->{waiting}=undef;
	unless (defined $result) { stop($self,_"connection failed."); return; }
	my @list;
	while ($result=~m#<img src=([^>]*?) width=\d+ height=\d+>(?:.*?imgurl=([^>]*?)&)?#g)
	{	my $url='http://images.google.com'.$1;
		$url= $2? [$2,$url] : $url;
		push @list, $url;
	}
	unless (@list) { stop($self,_"no matches found, you might want to remove some search terms."); return; }
	$self->{total}=@list;
	$self->{progress}->set_fraction(0);
	$self->{progress}->show;
	#$table->set_size_request(110*5,110*int(1+@list/5));
	$self->{toload}=\@list;
	::IdleDo('8_FetchCovers'.$self,100,\&get_next,$self);
}

sub stop
{	my ($self,$error)=@_;
	$self->{Bstop}->set_sensitive(0);
	$self->{waiting}->abort if $self->{waiting};
	$self->{waiting}=undef;
#	delete $::ToDo{'8_FetchCovers'.$self};
	#$self->{progress}->set_fraction(1);
	$self->{progress}->hide;
	if ($error)
	{	my $l=Gtk2::Label->new($error);
		$l->show;
		$self->{table}->attach($l,0,5,0,1,'fill','fill',1,1);
	}
}

sub get_next
{	my $self=shift;
	my $searchid=$self->{searchid};
	my $url=shift @{ $self->{toload} };
	unless ($url) { stop($self);return; }
	my $url_bis;
	if (ref $url) {($url,$url_bis)=@$url}
	$self->{waiting}=Simple_http::get_with_cb(url => $url, cache=>1, cb =>
	sub
	{	my $pixdata=$_[0];
		$self->{waiting}=undef;
		my $loader;
		$loader=::LoadPixData($pixdata,100) if $pixdata;
		if ($loader)
		{	my $dim=$loader->{w}.' x '.$loader->{h};
			my $table=$self->{table};
			my $pixbuf=$loader->get_pixbuf;
			my $image=Gtk2::Image->new_from_pixbuf($pixbuf);
			my $button=Gtk2::Button->new;
			$button->{pixdata}=$pixdata;
			$button->{ext}=	($Gtk2::VERSION >= 1.092)?
					  $loader->get_format->{extensions}[0]
					: ( EntryCover::_identify_pictype($pixdata) )[0];
			#$button->{url}=$url;
			$button->add($image);
			$::Tooltips->set_tip($button,$dim."\n".$url);
			$button->signal_connect(clicked => \&set_cover);
			$button->signal_connect(button_press_event => \&::pixbox_button_press_cb,3); # 3 : mouse button 3
			$button->set_relief('none');
			$button->show_all;
			my $x=$table->{x}||0; my $y=$table->{y}||0;
			$table->attach($button,$x,$x+1,$y,$y+1,'fill','fill',1,1);
			$x++; if ($x>4) {$x=0; $y++;}
			$table->{x}=$x; $table->{y}=$y;
		}
		elsif ($url_bis) {unshift @{ $self->{toload} },$url_bis; }
		$self->{progress}->set_fraction(1- @{$self->{toload}} / $self->{total} );
		::IdleDo('8_FetchCovers'.$self,100,\&get_next,$self);
	});
}

sub set_cover
{	my $button=$_[0];
	my $self=::find_ancestor($button,__PACKAGE__);
	my $table=$self->{table};
	my $album=$table->{album};
	my $check=Gtk2::CheckButton->new("use this picture as cover for album '$album'");
	$check->set_active(1);
	my $default_file=	$::Options{'PLUGIN_'.PID.'_USEFILE'} ?
			$::Options{'PLUGIN_'.PID.'_COVERFILE'} : $album;
	$default_file=~s/\.(?:jpg|png|gif)$//;
	$default_file.='.'.$button->{ext};
	$default_file=~s/$::ILLEGALCHAR//g;
	$default_file=::filename_from_unicode($default_file);
	my $default_dir=$::Options{'PLUGIN_'.PID.'_COVERPATH'} || '';
	$default_dir=::filename_from_unicode($default_dir);
	$default_dir=~s/$::ILLEGALCHARDIR//g;
	$default_dir=$table->{dir} unless $::Options{'PLUGIN_'.PID.'_USEPATH'} && -d $default_dir;
	if ($::Options{'PLUGIN_'.PID.'_UNIQUE'})
	{	while (-e $default_dir.::SLASH.$default_file) #find a unique name
		{ last unless $default_file=~s/(?:_(\d+))?\.(\w+)$/'_'.($1? $1+1 : 1).".$2"/e ; }
	}
	my $file=$default_dir.::SLASH.$default_file;
	if (!$::Options{'PLUGIN_'.PID.'_ASK'} || -e $file)
	{$file=::ChooseSaveFile($self,_"Save picture as",
		$default_dir,$default_file,
		$check);
	}
	return unless $file;
	my $fh;
	until (open $fh,'>',$file)
	{	my $retry=::Retry_Dialog("Can't open ".::filename_to_utf8displayname($file)." for writing : .$!",$self);
		return unless $retry eq 'yes';
	}
	print $fh $button->{pixdata};
	close $fh;
	return unless $check->get_active;
	$::Album{$album}[::AAPIXLIST]=$file;
	::HasChanged('AAPicture',$album);
}

1

__END__
xml.amazon.com/onca/xml3?t=webservices-20&dev-t=%l&KeywordSearch=%s&mode=music&type=heavy&locale=us&page=1&f=xml
