package GMB::Plugin::Lyrics;
use strict;
use warnings;
require 'simple_http.pm';
use base 'Gtk2::VBox';
use constant
{	PID => 'LYRICS',#used to identify the plugin and its options
	name => _"Lyrics",
	title => _"Lyrics",
};

$::Plugins{&PID}=__PACKAGE__;

my $site=0;
my @sites=
(	['http://lyrc.com.ar/en/tema1en.php','artist=%a&songname=%s'],
	#['http://api.leoslyrics.com/api_search.php?artist=%a&songtitle=%s'],
	#['http://www.google.com/search?q=%a+%s'],
);

sub init
{	$::Context{Lyrics}=__PACKAGE__;
	::HasChanged('Context','add','Lyrics');
}
sub end
{	::HasChanged('Context','remove','Lyrics');
	delete $::Context{Lyrics};
}

sub new
{	my $class=shift;
	my $self = bless Gtk2::VBox->new, $class;
	my $textview=Gtk2::TextView->new;
	$textview->signal_connect(button_release_event => \&button_release_cb);
	$textview->signal_connect(motion_notify_event => \&update_cursor_cb);
	$textview->signal_connect(visibility_notify_event => \&update_cursor_cb);
	$textview->set_wrap_mode('word');
	$self->{buffer}=$textview->get_buffer;
	my $sw=Gtk2::ScrolledWindow->new(undef, undef);
	$sw->set_shadow_type('etched-in');
	$sw->set_policy('automatic', 'automatic');
	$sw->add_with_viewport($textview);
	$self->add($sw);
	$self->show_all;
	$self->signal_connect(destroy => \&destroy_event_cb);
	return $self;
}
sub destroy_event_cb
{	my $self=shift;
	$self->cancel;
}

sub cancel
{	my $self=shift;
	delete $::ToDo{'8_lyrics'.$self};
	$self->{waiting}->abort if $self->{waiting};
	$self->{waiting}=$self->{pixtoload}=undef;
}

sub prefbox
{	my $vbox=Gtk2::VBox->new(::FALSE, 2);
	my $label1=Gtk2::Label->new(_"Lyrics plugin");
	my $label2=Gtk2::Label->new(_"Display lyrics of the current song using lyrc.com.ar");
	my $Bopen=Gtk2::Button->new(_"open context window");
	$Bopen->signal_connect(clicked => sub { ::ContextWindow; });
	$vbox->pack_start($_,::FALSE,::FALSE,1) for $label1,$label2,$Bopen;
	return $vbox;
}

sub SongChanged
{	my ($self,$ID)=@_;
	return unless defined $ID;
	return if defined $self->{ID} && $ID==$self->{ID};
	$self->{ID}=$ID;
	my $ref=$::Songs[$ID];
	my $title=$ref->[::SONG_TITLE];
	my $artist=$ref->[::SONG_ARTIST];
	s/([^A-Za-z0-9-])/sprintf('%%%02X', ord($1))/eg for $title,$artist;
	my ($url,$post)=@{$sites[$site]};
	for ($url,$post) { next unless defined $_; s/%a/$artist/; s/%s/$title/; }
	#$self->load_url($url,$post);
	::IdleDo('8_lyrics'.$self,1000,\&load_url,$self,$url,$post);
}

sub load_url
{	my ($self,$url,$post)=@_;
	$self->{buffer}->set_text(_"Loading...");
	$self->{url}=$url;
	$self->{post}=$post;
	$self->cancel;
	$self->{waiting}=Simple_http::get_with_cb(cb => sub {$self->loaded(@_)},url => $self->{url},post => $self->{post});
}

sub loaded
{	my $self=shift;
	my $buffer=$self->{buffer};
	my ($data,$type)=($_[0],$_[1]);
	unless ($data) { $data=_("Loading failed.").' <a href="'.$self->{url}.'">'._("retry").'</a>'; $type="text/html"; }
	$buffer->delete($buffer->get_bounds);
	my $encoding;
	if ($type && $type=~m#^text/.*; ?charset=([\w-])#) {$encoding=$1}
	if ($type && $type!~m#^text/html#)
	{	if	($type=~m#^text/#)	{$buffer->set_text($data);}
		elsif	($type=~m#^image/#)
		{	my $loader=::LoadPixData($data);
			if (my $p=$loader->get_pixbuf) {$buffer->insert_pixbuf(0,$p);}
		}
		return;
	}
	$encoding=$1 if $data=~m#<meta *http-equiv="Content-Type" *content="text/html; charset=([\w-]+)"#;
	$encoding='cp1252' if $encoding && $encoding eq 'iso-8859-1'; #microsoft use the superset cp1252 of iso-8859-1 but says it's iso-8859-1	Is this the right way to do it ?
	$data=Encode::decode($encoding,$data) if $encoding;
	for ($data)
	{s/<!--.*?-->//g;
	 s/[\r\n]//g;
	 s#<br(?: /)?>#\n#g;
	 s#<p>#\n\n#g;
	 s#&nbsp;#  #g;
	}

	my @l=split /(<.*?>)/s, $data;
	my (%prop,$ul,@urls,@pixbufs,$title);
	while (defined($_=shift @l))
	{	my $iter=($buffer->get_bounds)[1];
		if (m/^[^<]/)
		{	$_=(''x$ul).'- '.$_ if $ul;
			s/&#(\d{2,4});/chr($1)/eg;
			if (keys %prop)
			{ my $tag=$buffer->create_tag(undef,%prop);
			  $buffer->insert_with_tags($iter, $_, $tag);
			}
			else	{$buffer->insert($iter, $_);}
		}
		elsif (my ($tag,$p)=m#^<(\w+) (.*)>$#)
		{	my %p;
			$p{$1}=$3 while $p=~m/(\w+)=(["'])(.+?)\2/sg;
			if ($tag eq 'a' && exists $p{href})
				{ push @urls,$iter->get_offset,$p{href}; }
			elsif ($tag eq 'script' || $tag eq 'style')
				{shift @l while $l[0] ne "</$tag>"}
			elsif ($tag eq 'font')
				{ if ($p{size} && $p{size}=~m/^\d/)
				  {	$prop{size}=$p{size}*8*Gtk2::Pango::PANGO_SCALE }
				  else {delete $prop{size};}
				}
			elsif ($tag=~m/^h(\d)/)
				{ $prop{size}=(20,15,12)[$1]*Gtk2::Pango::PANGO_SCALE;
				  $prop{weight}=Gtk2::Pango::PANGO_WEIGHT_BOLD if $1 eq '1';
				}

			if ($tag eq 'img' && exists $p{src})
			{	push @pixbufs,[$iter->get_offset,$p{src}];
			}
		}
		elsif (m#^</(\w+)>$#)
		{	if    ($1 eq 'a')	{push @urls,$iter->get_offset if @urls%3==2}
			elsif ($1 eq 'u')	{delete $prop{underline};}
			elsif ($1 eq 'b')	{delete $prop{weight};}
			elsif ($1 eq 'font')	{delete $prop{size};}
			elsif ($1=~m/^h(\d)/)	{delete $prop{size};delete $prop{weight};}
			elsif ($1 eq 'ul')	{$ul--;}
		}
		elsif (m#^<(\w+)>$#)
		{	if    ($1 eq 'u')	{$prop{underline}='single';}
			elsif ($1 eq 'b')	{$prop{weight}=Gtk2::Pango::PANGO_WEIGHT_BOLD;}
			elsif ($1 eq 'title')	{$title=shift @l while $l[0] ne "</$1>"}
			elsif ($1 eq 'ul')	{$ul++}
		}
	}
	while (@urls)
	{	my ($offs1,$url,$offs2)=splice @urls,0,3; warn "($offs1,$url,$offs2)\n" if $::debug;
		my $tag=$buffer->create_tag(undef,foreground => 'blue',underline => 'single');
		if ($url eq '#') {$tag=$buffer->create_tag(undef,invisible => 1);}
		$tag->{url}=$url;
		$buffer->apply_tag($tag,
			$buffer->get_iter_at_offset($offs1),
			$buffer->get_iter_at_offset($offs2));
	}
	$self->{pixtoload}=\@pixbufs;
	::IdleDo('8_FetchPix'.$self,100,\&load_pixbuf,$self) if @pixbufs;
}

sub load_pixbuf
{	my $self=shift;
	my $ref=pop @{ $self->{pixtoload} };
	return 0 unless $ref;
	my ($offs,$url)=@$ref;
	$self->{waiting}=Simple_http::get_with_cb(url => $self->full_url($url), cache=>1, cb=>
	sub
	{	$self->{waiting}=undef;
		my $loader;
		$loader=::LoadPixData($_[0]) if $_[0];
		if ($loader)
		{	my $buffer=$self->{buffer};
			$buffer->insert_pixbuf($buffer->get_iter_at_offset($offs),$loader->get_pixbuf);
		}
		::IdleDo('8_FetchPix'.$self,100,\&load_pixbuf,$self);
	});
#::IdleDo('8_FetchPix'.$self,100,\&load_pixbuf,$self) unless $self->{waiting};
}

sub loaded_old
{	my $self=shift;
	my $buffer=$self->{buffer};
	unless ($_[0]) {$buffer->delete($buffer->get_bounds);$buffer->set_text(_"Loading failed.");return;}
	local $_=$_[0];
	s/[\r\n]//g;
	s#<title>.*?</title>##;
	s#<script .*?</script>##g;
	s#<a href="([^"]+)">(.*?)</a>#<>$2<>$1<>#g;
	s#<br(?: /)?>#\n#g;
	s/<[^>]+>//g;
	s/^\n+//;
	s/BADSONG\n+$//;

	$buffer->delete($buffer->get_bounds);
	my @l=split /(<>)/,$_;
	while (defined ($_=shift @l))
	{	my $iter=($buffer->get_bounds)[1];
		if ($_ eq '<>')
		{	my ($text,undef,$url,undef)=splice @l,0,4;
			my $tag=$buffer->create_tag(undef,foreground => 'blue',underline => 'single');
			$tag->{url}=$url;
			$buffer->insert_with_tags($iter, $text, $tag);
		}
		else { $buffer->insert($iter, $_);}
	}
}

sub button_release_cb
{	my ($textview,$event) = @_;
	my $self=::find_ancestor($textview,__PACKAGE__);
	return ::FALSE unless $event->button == 1;
	my ($x,$y)=$textview->window_to_buffer_coords('widget',$event->x, $event->y);
	my $iter=$textview->get_iter_at_location($x,$y);
	$self->follow_if_link($iter);
	return ::FALSE;
}

sub follow_if_link
{	my ($self,$iter)=@_;
	for my $tag ($iter->get_tags)
	{	next unless $tag->{url};
		last if $tag->{url}=~m/^#/; #FIXME
		my $url= $self->full_url( $tag->{url} );
		if ($url=~m#^http://www\.lyrc\.com\.ar/en/add/add\.php\?#) {::openurl( quotemeta($url) ); return}	#lyrc specific
		$self->load_url($url);
		last;
	}
}

sub full_url
{	my ($self,$url)=@_;
	return $url if $url=~m#^http://#;
	my $base=$self->{url};
	if ($url=~m#^/#){$base=~s#^(?:http://)?([^/]+).*$#$1#;}
	else		{$base=~s#[^/]*$##;}
	return $base.$url;
}


sub update_cursor_cb
{	my $textview=$_[0];
	my (undef,$wx,$wy,undef)=$textview->window->get_pointer;
	my ($x,$y)=$textview->window_to_buffer_coords('widget',$wx,$wy);
	my $buffer=$textview->get_buffer;
	my $iter=$textview->get_iter_at_location($x,$y);
	my $cursor;
	for my $tag ($iter->get_tags)
	{	next unless $tag->{url};
		$cursor=1;
		last;
	}
	if	($textview->{cursor} && !$cursor) { $textview->{cursor}=undef;	}
	elsif	(!$textview->{cursor} && $cursor) { $textview->{cursor}=1;	}
	else {return}
	$cursor= Gtk2::Gdk::Cursor->new( $cursor? 'hand2' : 'xterm' ); #FIXME perhaps it should create a new cursor everytime ?
	$textview->get_window('text')->set_cursor($cursor);
}

1
