package GMB::Plugin::Moz;
use strict;
use warnings;
require 'simple_http.pm';
use Gtk2::MozEmbed;
use base 'Gtk2::VBox';
use constant
{	PID => 'MozEmbed',#used to identify the plugin and its options
	name => 'MozEmbed',
};

$::Plugins{&PID}=__PACKAGE__;
Gtk2::MozEmbed->set_profile_path ($::HomeDir,'mozilla_profile');
our $Embed=Gtk2::MozEmbed->new;#needed to keep a Gtk2::MozEmbed to prevent xpcom from shutting down with the last gtkmozembed, should use gtk_moz_embed_push_startup and gtk_moz_embed_pop_startup but not bound yet

my $active;
::SetDefaultOptions(PID, StrippedWiki => 1);

sub init
{	$active=1;
	update_Context_hash();
	&set_stripped_wiki;
}
sub end
{	$active=0;
	update_Context_hash(1);
}

sub update_Context_hash
{	my $lyrics= !$::Options{'PLUGIN_'.PID.'_DisableLyrics'} && $active;
	if (exists $::Context{MozLyrics})
	{	if (!$lyrics)
		{	::HasChanged('Context','remove','MozLyrics');
			delete $::Context{MozLyrics};
		}
	}
	elsif ($lyrics)
	{	$::Context{MozLyrics}='GMB::Plugin::Moz::Lyrics';
		::HasChanged('Context','add','MozLyrics');
	}

	my $wiki= !$::Options{'PLUGIN_'.PID.'_DisableWiki'} && $active;
	if (exists $::Context{MozWikipedia})
	{	if (!$wiki)
		{	::HasChanged('Context','remove','MozWikipedia');
			delete $::Context{MozWikipedia};
		}
	}
	elsif ($wiki)
	{	$::Context{MozWikipedia}='GMB::Plugin::Moz::Wikipedia';
		::HasChanged('Context','add','MozWikipedia');
	}

}

sub set_stripped_wiki
{	my $content='';
	if ($::Options{'PLUGIN_'.PID.'_StrippedWiki'})
	{ $content="/*@-moz-document domain(wikipedia.org) { */
.portlet {display: none !important;}
#f-list {display: none !important;}
#footer {display: none !important;}
#content {margin: 0 0 0 0 !important; padding: 0 0 0 0 !important}
/* } */	";
	}
	open my $fh,'>',join(::SLASH,$::HomeDir,'mozilla_profile','chrome','userContent.css');
	print $fh $content;
	close $fh;
}

sub new
{	my ($class)=@_;
	my $self = bless Gtk2::VBox->new, $class;
	my $hbox = Gtk2::HBox->new;
	my $status=$self->{status}=Gtk2::Statusbar->new;
	$status->{id}=$status->get_context_id('link');
	my $embed=$self->{embed}=Gtk2::MozEmbed->new;
	my $back= $self->{BBack}=Gtk2::ToolButton->new_from_stock('gtk-go-back');
	my $next= $self->{BNext}=Gtk2::ToolButton->new_from_stock('gtk-go-forward');
	my $stop= $self->{BStop}=Gtk2::ToolButton->new_from_stock('gtk-stop');
	$self->{$_}->set_sensitive(0) for qw/BBack BNext BStop/;
	$hbox->pack_start($_,::FALSE,::FALSE,4) for $back,$next,$stop,$self->addtoolbar;
	$self->pack_start($hbox,::FALSE,::FALSE,1);
	$self->add($embed);
	$self->pack_end($status,::FALSE,::FALSE,1) if $::Options{'PLUGIN_'.PID.'_StatusBar'};
	$self->show_all;
	$back->signal_connect(clicked => sub { $embed->go_back });
	$next->signal_connect(clicked => sub { $embed->go_forward });
	$stop->signal_connect(clicked => sub { $embed->stop_load });
	$embed->signal_connect(link_message => \&link_message_cb);
	$embed->signal_connect(net_stop => \&net_startstop_cb,0);
	$embed->signal_connect(net_start => \&net_startstop_cb,1);
	return $self;
}

sub prefbox
{	my $vbox=Gtk2::VBox->new(::FALSE, 2);
	my $label1=Gtk2::Label->new(_"Mozilla plugin");
	my $label2=Gtk2::Label->new(_"provide context views using MozEmbed");
	#my $combo=::NewPrefCombo('PLUGIN_'.PID.'_Site',[sort keys %sites],'site : ',sub {$ID=undef;&Changed;});
	my $check=::NewPrefCheckButton('PLUGIN_'.PID.'_StrippedWiki',_"Strip wikipedia pages",\&set_stripped_wiki,_"Remove header, footer and left column from wikipedia pages");
	my $Bopen=Gtk2::Button->new(_"open context window");
	$Bopen->signal_connect(clicked => sub { ::ContextWindow; });
	my $check1=::NewPrefCheckButton('PLUGIN_'.PID.'_DisableWiki',_"Disable wikipedia context tab",\&update_Context_hash);
	my $check2=::NewPrefCheckButton('PLUGIN_'.PID.'_DisableLyrics',_"Disable Lyrics context tab",\&update_Context_hash);
	$vbox->pack_start($_,::FALSE,::FALSE,1) for $label1,$label2,$check1,$check2,$check,$Bopen;
	return $vbox;
}

sub load_url
{	my ($self,$url,$post)=@_;
	$self->{url}=$url;
	$self->{post}=$post;
	if ($post)
	{ Simple_http::get_with_cb(cb => sub {$self->loaded(@_)},url => $url,post => $post); }
	else {$self->{embed}->load_url($url);}
}

sub loaded
{	my ($self,$data,$type)=@_;
	my $embed=$self->{embed};
	$embed->render_data($data,$self->{url},$type)
#	$embed->open_stream ($self->{url}, $type);
#	$embed->append_data ($data);
#	$embed->close_stream;
}

sub link_message_cb
{	my $embed=$_[0];
	my $self=::find_ancestor($embed,__PACKAGE__);
	my $statusbar=$self->{status};
	$statusbar->pop( $statusbar->{id} );
	$statusbar->push( $statusbar->{id}, $embed->get_link_message );
}

sub net_startstop_cb
{	my ($embed,$loading)=@_;
	my $self=::find_ancestor($embed,__PACKAGE__);
	$self->{BStop}->set_sensitive( $loading );
	$self->{BBack}->set_sensitive( $embed->can_go_back );
	$self->{BNext}->set_sensitive( $embed->can_go_forward );
}

package GMB::Plugin::Moz::Lyrics;
our @ISA=('GMB::Plugin::Moz');

use constant
{	title => _"Lyrics",
	PID => GMB::Plugin::Moz::PID,  #FIXME
};

my %sites=
(	#lyrc => ['lyrc','http://lyrc.com.ar/en/tema1en.php','artist=%a&songname=%s'],
	lyrc => ['lyrc','http://lyrc.com.ar/en/tema1en.php?artist=%a&songname=%s'],
	#leoslyrics => ['leolyrics','http://api.leoslyrics.com/api_search.php?artist=%a&songtitle=%s'],
	google  => ['google','http://www.google.com/search?q="%a"+"%s"'],
	googlemusic  => ['google music','http://www.google.com/musicsearch?q="%a"+"%s"'],
	lyriki  => ['lyriki','http://lyriki.com/index.php?title=%a:%s'],
);

::SetDefaultOptions(PID, LyricSite => 'google');

sub addtoolbar
{	my $self=$_[0];
	my $combo=Gtk2::ComboBox->new_text;
	my $n=0; my $found;
	for my $site (sort keys %sites)
	{	$combo->append_text($sites{$site}[0]);
		$found=$n if $::Options{'PLUGIN_'.PID.'_LyricSite'} eq $site;
		$n++;
	}
	$combo->set_active($found||0);
	$combo->signal_connect(changed => sub
		{	my $n=$_[0]->get_active;
			$::Options{'PLUGIN_'.PID.'_LyricSite'}= (sort keys %sites)[$n];
			$self->SongChanged($self->{ID},1);
		});
	return $combo;
}

sub SongChanged
{	my ($self,$ID,$force)=@_;
	return unless defined $ID;
	return if defined $self->{ID} && $ID==$self->{ID} && !$force;
	$self->{ID}=$ID;
	my $ref=$::Songs[$ID];
	my $title=$ref->[::SONG_TITLE];
	return if !defined $title || $title eq '';
	my $artist=$ref->[::SONG_ARTIST];
	s/([^A-Za-z0-9-])/sprintf('%%%02X', ord($1))/eg for $title,$artist;
	my (undef,$url,$post)=@{$sites{$::Options{'PLUGIN_'.PID.'_LyricSite'}}};
	for ($url,$post) { next unless defined $_; s/%a/$artist/; s/%s/$title/; }
	::IdleDo('8_mozlyrics'.$self,1000,sub {$self->load_url($url,$post)});
}

package GMB::Plugin::Moz::Wikipedia;
our @ISA=('GMB::Plugin::Moz');

use constant
{	title => _"Wikipedia",
	PID => GMB::Plugin::Moz::PID,  #FIXME
};

my %locales=
(	en => 'English',
	fr => 'Franais',
	de => 'Deutsch',
	pl => 'Polski',
	nl => 'Nederlands',
	sv => 'Svenska',
	it => 'Italiano',
	pt => 'Portugus',
	es => 'Espaol',
#	ja => "\xe6\x97\xa5\xe6\x9c\xac\xe8\xaa\x9e\x0a",
);
#::_utf8_on( $locales{ja} );

::SetDefaultOptions(PID, WikiLocale => 'en');

sub addtoolbar
{	my $self=$_[0];
	my $combo=Gtk2::ComboBox->new_text;
	my $n=0; my $found;
	for my $loc (sort {$locales{$a} cmp $locales{$b}} keys %locales)
	{	$combo->append_text($locales{$loc});
		$found=$n if $::Options{'PLUGIN_'.PID.'_WikiLocale'} eq $loc;
		$n++;
	}
	$combo->set_active($found||0);
	$combo->signal_connect(changed => sub
		{	my $n=$_[0]->get_active;
			$::Options{'PLUGIN_'.PID.'_WikiLocale'}= (sort {$locales{$a} cmp $locales{$b}} keys %locales)[$n];
			$self->SongChanged($self->{ID},1);
		});
	return $combo;
}

sub SongChanged
{	my ($self,$ID,$force)=@_;
	return unless defined $ID;
	$self->{ID}=$ID;
	my $artist=$::Songs[$ID][::SONG_ARTIST];
	return if !defined $artist || $artist eq '' || $artist eq '<Unknown>';
	return if defined $self->{Artist} && $artist eq $self->{Artist} && !$force;
	$self->{Artist}=$artist;
	$artist=~s/([^A-Za-z0-9-])/sprintf('%%%02X', ord($1))/eg;
	my $url='http://'.$::Options{'PLUGIN_'.PID.'_WikiLocale'}.'.wikipedia.org/wiki/'.$artist;
	::IdleDo('8_mozpedia'.$self,1000,sub {$self->load_url($url)});
	#::IdleDo('8_mozpedia'.$self,1000,sub {$self->wikiload});
}

sub wikiload	#not used for now
{	my $self=$_[0];
	my $url=$self->{Artist};
	$url=~s/([^A-Za-z0-9-])/sprintf('%%%02X', ord($1))/eg;
	$url='http://'.$::Options{'PLUGIN_'.PID.'_WikiLocale'}.'.wikipedia.org/wiki/'.$url;
	#$url='http://google.com/search?q='.$url;
	$self->{url}=$url;
	Simple_http::get_with_cb(cb => sub
		{	my $cb=sub { $self->wikifilter(@_) };
			if (!$_[0] || $_[0]=~m/No page with that title exists/)
			{	Simple_http::get_with_cb(cb => $cb, url => $url);
			}
			else { $self->{url}.='_(band)'; goto $cb }
		},url => $url.'_(band)');
}

sub wikifilter	#not used for now
{	my ($self,$data,$type)=@_;
	return unless $data;	#FIXME
	#$data='<style type="text/css">.firstHeading {display: none}</style>'.$data;
	$self->loaded($data,$type);
}

1

# FIXME use google music : http://www.google.com/musicsearch?q="%a"
