die "Plugin with the same name already loaded\n" if $::Plugins{NOWPLAYING};
package GMB::Plugin::NowPlaying;
use strict;
use warnings;
use constant
{	PID	=> 'NOWPLAYING',
	name	=> _"Now playing",
};
my $handle;
my $lasttime;

$::Plugins{&PID}=__PACKAGE__;	#register the plugin

# the plugin package must have these sub :
# init	: called when the plugin is activated
# end	: called when the plugin is de-activated
# name	: returns the name of the plugin (displayed in the plugin list)
# PID	: returns the Plugin ID, used to identify the plugin and its options
# prefbox : returns a Gtk2::Widget used to describe the plugin and set its options

sub init
{	$handle={};	#the handle to the Watch function must be a hash ref, if it is a Gtk2::Widget, UnWatch will be called when the widget is destroyed
	::Watch($handle,'SongID',\&Changed);
	::Watch($handle,'Playing',\&Changed);
}
sub end
{	::UnWatch($handle,'SongID');
	::UnWatch($handle,'Playing');
}

sub prefbox
{	my $vbox=Gtk2::VBox->new(::FALSE, 2);
	my $label1=Gtk2::Label->new(_"NowPlaying plugin");
	my $label2=Gtk2::Label->new(_"run a command when playing a song");
	$label2->set_line_wrap(::TRUE);
	my $entry=::NewPrefEntry('PLUGIN_'.PID.'_CMD',_"command :");
	my $check=::NewPrefCheckButton('PLUGIN_'.PID.'_SENDSTDINPUT',_"Send Title/Artist/Album in standard input");
	my $replacetable=::MakeReplaceTable('talydnfc');
	$vbox->pack_start($_,::FALSE,::FALSE,2) for $label1,$label2,$replacetable,$entry,$check;
	return $vbox;
}

sub Changed
{	return unless defined $::SongID && $::TogPlay;
	return if $lasttime && $::StartTime==$lasttime; #if song hasn't really changed
	$lasttime=$::StartTime;
	my $ID=$::SongID;
	my @cmd=split / /,$::Options{'PLUGIN_'.PID.'_CMD'};
	return unless @cmd;
	$_=::ReplaceFields($ID,$_) for @cmd;
	if ($::Options{'PLUGIN_'.PID.'_SENDSTDINPUT'})
	{	my $ref=$::Songs[$ID];
		my $string=	'Title='.	$ref->[::SONG_TITLE]."\n"
				.'Artist='.	$ref->[::SONG_ARTIST]."\n"
				.'Album='.	$ref->[::SONG_ALBUM]."\n"
		;
		open my$out,'|-',@cmd;
		print $out $string;
		close $out;
	}
	else
	{	system @cmd;
	}
}

1 #the file must return true

# song properties :
# SONG_UFILE	SONG_UPATH	SONG_MODIF
# SONG_LENGTH	SONG_SIZE	SONG_BITRATE
# SONG_FORMAT	SONG_CHANNELS	SONG_SAMPRATE
# SONG_TITLE	SONG_ARTIST	SONG_ALBUM
# SONG_DISC	SONG_TRACK	SONG_DATE
# SONG_VERSION	SONG_GENRE	SONG_COMMENT
# SONG_AUTHOR
# SONG_ADDED	SONG_LASTPLAY	SONG_NBPLAY
# SONG_RATING	SONG_FLAGS
# SONG_FILE	SONG_PATH
#
# SONG_GENRE & SONG_FLAGS are "\x00" separated lists
#
# filename of album cover is in $::Album{ $::Songs[$::SongID][::SONG_ALBUM] }[::AAPIXLIST]
#  (may be an mp3 file, '0' means "no cover and don't auto-set cover")
# if you change the album cover, you should call : ::HasChanged('AAPicture',$album); where $album is the album name
